#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on 07/04/2025 15:01:05

@author: psakic
"""

import logging.config
import os
from os import path

__version__ = "2.4.0"  #  changed automaticcaly with bump-my-version

#### IMPORT CONFIG FOR LOGGER
log_file_path = os.path.join(
    path.dirname(path.abspath(__file__)), "cfglog", "cfglog.py"
)

if os.path.isfile(log_file_path):
    from . import cfglog

    logging.config.dictConfig(cfglog.log_config_dict)
else:
    print("ERR:logger cfgfiles file", log_file_path, "is missing")


#### IMPORT AUTORINO INTERNAL SUBMODULES
from . import api
from . import bin

# from . import cfgenv
# from . import cfgfiles
# from . import cfglog
# from . import check
# from . import common
# from . import convert
# from . import download
# from . import handle

__all__ = [
    "api",
    "bin",
    # 'cfgenv',
    # 'cfgfiles',
    # 'cfglog',
    # 'check',
    # 'common',
    # 'convert',
    # 'download',
    # 'handle'
]

if __name__ == "__main__":
    print("autorino version", __version__)
    print("autorino.__all__", __all__)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on 29/04/2025 12:18:06

@author: psakic
"""

import autorino.cfgfiles as arocfg
import argparse
import yaml

def main():
    parser = argparse.ArgumentParser(
        description="Check autorino configuration file",
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )

    parser.add_argument(
        "-c",
        "--config",
        type=str,
        help="The input site configuration file or directory of sites configuration files. "
        "If a directory is provided, all files ending with '.yml' will be used.",
        required=True,
    )
    parser.add_argument(
        "-i",
        "--include_config",
        type=str,
        nargs="+",
        help="The main configuration file to be used.",
        default=None,
    )

    parser.add_argument(
        "-o",
        "--output",
        type=str,
        help="The output path where to write the checked configuration.",
        default=None,
    )

    args = parser.parse_args()

    _ , _ , yout= arocfg.read_cfg(site_cfg_path=args.config,
                    include_cfg_paths_xtra=args.include_config,
                    verbose=True)

    if args.output is not None:
        yaml.dump(yout, open(args.output, "w+"))


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on 30/05/2024 16:22:55

@author: psakic
"""

import argparse
import os.path

import yaml
import autorino.api as aroapi

def main():
    parser = argparse.ArgumentParser(description="Convert RAW files to RINEX.")
    parser.add_argument("-i",
        "--inp_raws",
        required=True,
        nargs="+",
        help="The input RAW files to be converted "
        "Possible inputs are: \n"
        "* one single RAW file path \n"
        "* a comma-separated (,) list of RAW paths \n"
        "* a text file containing a list of RAW paths \n"
        "(then --list_file_input must be activated) \n"
        "* a directory containing RAW files \n",
    )
    parser.add_argument("-o",
        "--out_dir",
        help="The output directory where the converted files will be stored",
        required=True
    )

    parser.add_argument(
        "-t",
        "--out_structure",
        help="The structure of the output directory. "
             "If provided, the converted files will be "
             "stored in a subdirectory of out_dir following this structure. "
             "See README.md for more information. "
             "Typical values are '<SITE_ID4>/%%Y/' or '%%Y/%%j/ (default).",
        default="%Y/%j",
    )

    parser.add_argument(
        "-m",
        "--metadata",
        help="""
        The metadata to be included in the converted RINEX files. \n 
        Possible inputs are: \n 
        * list of string (sitelog file paths)  
        * single string (single sitelog file path) 
        * single string (directory containing the sitelogs)
        """,
    )
    parser.add_argument(
        "-frnx",
        "--force_rnx",
        action="store_true",
        help="Force the conversion even if the output files already exist",
        default=False,
    )

    parser.add_argument(
        "-fraw",
        "--force_raw",
        action="store_true",
        help="Force the RAW file archiving even if the output files already exist",
        default=False,
    )

    parser.add_argument(
        "-l",
        "--list_file_input",
        action="store_true",
        help="If set to True, the input RAW files are provided as a list in a text file",
        default=False,
    )

    parser.add_argument(
        "-tmp",
        "--tmp_dir",
        help="The temporary directory used during the conversion process. "
             "If not provided, it defaults to <$HOME>/autorino_workflow/tmp. ",
        default='<$HOME>/autorino_workflow/tmp',
    )
    parser.add_argument(
        "-log",
        "--log_dir",
        help="The directory where logs will be stored. "
             "If not provided, it defaults to <$HOME>/autorino_workflow/log",
        default='<$HOME>/autorino_workflow/log',
    )
    parser.add_argument(
        "-rimo",
        "--rinexmod_options",
        type=yaml.safe_load,
        help="The options for modifying the RINEX files during the conversion. "
             "The options must be provided in a dictionnary represented as a string "
             "e.g. '{longname: False, filename_style: basic}' "
             "Defaults to None",
        default=None,
    )

    parser.add_argument(
        "-ro",
        "--raw_out_dir",
        help="Directory where RAW files will be archived. "
             "No deletion will occur, your RAW files are sacred.",
        default=None,
    )

    parser.add_argument(
        "-rt",
        "--raw_out_structure",
        help="Structure for archiving RAW files. "
             "Defaults to the same structure as the output directory (-o/--out_dir) if not provided.",
        default=None,
    )

    parser.add_argument(
        "-p",
        "--processes",
        type=int,
        help="Number of processes to use for conversion. "
             "Defaults to 1 (single process).",
        default=1,
    )

    parser.add_argument(
        "-fpt",
        "--filter_prev_tables",
        action="store_true",
        help="If set, filters and skips previously converted files "
             "with tables stored in the tmp tables directory.",
        default=False,
    )

    parser.add_argument(
        "-c",
        "--converter",
        help="The software to be used for converting the RAW files to RINEX. "
             "Default is 'auto' which lets autorino choose the best available converter based on RAW file extension."
             "Possible values are:"
             "* 'auto' (automatic choice based on the extension),"
             "* 'trm2rnx' (Trimble unofficial),"
             "* 't0xconvert' (Trimble official),"
             "* 'runpkr00' (Trimble legacy),"
             "* 'teqc' (legacy conversion & RINEX Handeling),"
             "* 'mdb2rinex' (Leica),"
             "* 'sbf2rin' (Septentrio),"
             "* 'convbin' (BINEX),"
             "* 'tps2rin' (Topcon),"
             "* 'converto' (RINEX Handeling),"
             "* 'gfzrnx' (RINEX Handeling)",
        default="auto",
    )


    args = parser.parse_args()

    aroapi.convert_rnx(
        inp_raws=_prep_raws_inp(args),
        out_dir=args.out_dir,
        out_structure=args.out_structure,
        tmp_dir=args.tmp_dir,
        log_dir=args.log_dir,
        rinexmod_options=args.rinexmod_options,
        metadata=args.metadata,
        force_rnx=args.force_rnx,
        force_raw=args.force_raw,
        raw_out_dir=args.raw_out_dir,
        raw_out_structure=args.raw_out_structure,
        processes=args.processes,
        filter_prev_tables=args.filter_prev_tables,
        converter=args.converter
    )


def _prep_raws_inp(args):
    """
    see also step_fcts.import_files
    """
    if args.list_file_input:
        ### input is a filelist of RINEXs => output is a list
        with open(args.inp_raws[0], "r") as f:
            return f.read().splitlines()
    elif len(args.inp_raws) == 1 and os.path.isdir(args.inp_raws[0]):
        ### input is a directory => output is the directory str
        return args.inp_raws[0]
    else:
        ### input is a single or several RINEXs => output is a list
        # (if one single RINEX file, then it is a singleton list)
        return args.inp_raws

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on 19/09/2024 16:02:51

@author: psakic
"""

import argparse
import autorino.check as arochk

#### Import the logger
import logging
import autorino.cfgenv.env_read as aroenv

logger = logging.getLogger("autorino")
logger.setLevel(aroenv.ARO_ENV_DIC["general"]["log_level"])


def main():
    """
    Shell function to call trimble_filelist_html with command-line arguments.
    """
    parser = argparse.ArgumentParser(
        description="Generate a list of Trimble files from a remote HTML directory.",
        epilog="Example: autorino_trimble_filelist CBEZ <URL_of_CBEZ> /output/directory 2024-01-01 2024-12-31'",
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument("site", type=str, help="The site identifier (4 characters).")
    parser.add_argument("hostname", type=str, help="The hostname of the remote server.")
    parser.add_argument(
        "output_dir",
        type=str,
        help="The directory where the downloaded HTML files will be saved.",
    )
    parser.add_argument("start_date", help="The start date for the file search.")
    parser.add_argument("end_date", help="The end date for the file search.")
    parser.add_argument("-p",
        "--period",
        type=str,
        default="1ME",
        help="The period for the file search, usually a month. Default is '1ME'.",
    )
    parser.add_argument("-t",
        "--structure",
        type=str,
        default="download/Internal/%Y%m",
        help=r"The directory structure on the remote server. Default is 'download/Internal/\%Y\%m'.",
    )

    args = parser.parse_args()

    arochk.trimble_filelist_html(
        site=args.site,
        hostname=args.hostname,
        output_dir=args.output_dir,
        start_date=args.start_date,
        end_date=args.end_date,
        period=args.period,
        structure=args.structure,
    )


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Apr  7 12:07:18 2023

@author: psakic
"""

import datetime as dt
import os
import re
from pathlib import Path
import dateutil
import docker
import pwd
import grp

### new rinexmod v4 import
import rinexmod.api as rimo_api
### old rinexmod import
#from rinexmod import rinexmod_api as rimo_api

#### Import the logger
import logging
import autorino.cfgenv.env_read as aroenv
import autorino.convert as arocnv

logger = logging.getLogger("autorino")
logger.setLevel(aroenv.ARO_ENV_DIC["general"]["log_level"])


def site_list_from_metadata(metadata_inp):
    """
    Extracts a list of site IDs from the provided metadata.

    This function takes either a list of metadata objects or a directory path containing metadata files,
    and returns a list of 4-character site IDs extracted from the metadata.

    Parameters
    ----------
    metadata_inp : str or list
    Possible inputs are:
     * list of string (sitelog file paths),
     * single string (single sitelog file path)
     * single string (directory containing the sitelogs)
     * list of MetaData objects
     * single MetaData object

    This function is mainly a wrapper of `rinexmod_api.metadata_input_manage`

    Returns
    -------
    list
        A list of 4-character site IDs extracted from the metadata.

    Notes
    -----
    If `metadata_inp` is a directory path, the function will attempt to read metadata files from the directory.
    """
    ###############################################
    ### read metadata
    if not type(metadata_inp) is list and os.path.isdir(metadata_inp):
        metadata = rimo_api.metadata_input_manage(metadata_inp, force=False)
    else:
        metadata = metadata_inp

    # get the site (4chars) as a list
    site4_list = [s.site_id4 for s in metadata]
    # get the site (9chars) as a list
    # site9_list = [s.site_id9 for s in metadata]

    site9_list = []

    return site4_list, site9_list


def site_search_from_list(fraw_inp, site_list_inp):
    """
    Searches for the correct site name of a raw file from a list of correct site names.

    This function takes a raw file with an approximate site name and a list of correct site names,
    and attempts to find the correct site name of the raw file. If no match is found, it defaults
    to the first 4 characters of the raw file name.

    Parameters
    ----------
    fraw_inp : Path
        The name of the raw file with an approximate site name.
    site_list_inp : list
        A list of correct 4 or 9-character site names.
        Only the 4 first characters will be considered.

    Returns
    -------
    str
        The correct site name of the raw file. If no match is found in the list,
        the function returns the first 4 characters of the raw file name.

    Notes
    -----
    The function performs a case-insensitive search.
    """

    site_out = None
    for s in site_list_inp:
        s4 = s[:4]
        if re.search(s4, fraw_inp.name, re.IGNORECASE):
            site_out = s
            break
    if not site_out:  # last chance, get the 4 1st chars of the raw file
        site_out = fraw_inp.name[:4]
    return site_out


def slct_conv_odd_f(fraw_inp, ext_excluded=None):
    """
    Identifies the right converter for a raw file with an unconventional extension, or excludes the file
    if its extension matches an excluded one.

    This function performs a high-level case matching to determine the appropriate converter for a raw file.
    If the file's extension matches one in the excluded list, the file is skipped.

    See also autorino.conv_cmd_run._convert_select
    for the regular converter selection

    Parameters
    ----------
    fraw_inp : Path
        The name of the raw file with an unconventional extension.
    ext_excluded : list, optional
        A list of file extensions to be excluded. If a file's extension matches one in this list, the file is skipped.
        Default is [".TG!$", ".DAT", ".Z", ".BCK", "^.[0-9]{3}$", "Trimble", ".ORIG"].

    Returns
    -------
    str
        The name of the appropriate converter for the raw file. If the file is to be skipped, returns None.

    Notes
    -----
    The function uses regular expressions to match file extensions.
    """
    if ext_excluded is None:
        ext_excluded = [
            ".TG!$",
            ".DAT",
            ".Z",
            ".BCK",
            "^.[0-9]{3}$",
            #".A$", # allowed .A files allowed as Septentrio
            "Trimble",
            ".ORIG",
        ]

    fraw = Path(fraw_inp)
    ext = fraw.suffix.upper()

    if not ext or len(ext) == 0:
        conve = "tps2rin"
    elif re.match(".M[0-9][0-9]", ext):
        conve = "mdb2rinex"
    else:
        conve = "auto"
        for ext_exl in ext_excluded:
            if re.match(ext_exl, ext):
                conve = None
                logger.warning(
                    "%s will be skipped, excluded extention %s", fraw.name, ext_exl
                )
                break

    return conve


def stop_old_docker(max_running_time=120):
    """
    Stops Docker containers that have been running for a specified amount of time.

    This function is useful for stopping long-running trm2rinex Docker containers.
    It iterates over all running Docker containers and stops any that have been
    running for longer than the specified maximum running time.

    Parameters
    ----------
    max_running_time : int, optional
        The maximum running time (in seconds) for a Docker container.
        Any container running longer than this will be stopped.
        Default is 120 seconds.

    Returns
    -------
    None

    Raises
    ------
    docker.errors.DockerException
        If permission is denied for Docker.

    Notes
    -----
    The function uses the Docker Python SDK to interact with Docker containers.
    """
    try:
        client = docker.from_env()
    except docker.errors.DockerException:
        logger.warning("Permission denied for Docker")
        return None
    containers = client.containers.list()

    for container in containers:
        # Calculate the time elapsed since the container was started
        started_at = container.attrs["State"]["StartedAt"]
        started_at = dateutil.parser.parse(started_at)
        elapsed_time = dt.datetime.now(dt.timezone.utc) - started_at

        if elapsed_time > dt.timedelta(seconds=max_running_time):
            container.stop()
            logger.warning(
                f"Stopped container {container.name} after {elapsed_time} seconds."
            )

    return None


def get_current_user_grp():
    """
    Retrieves the current user and group names.

    This function uses the `pwd` and `grp` modules to get the current user's
    username and the current group's name.

    Returns
    -------
    tuple
        A tuple containing the current user's username and the current group's name.
    """
    # Get the current user
    current_user = pwd.getpwuid(os.getuid()).pw_name
    # Get the current group
    current_group = grp.getgrgid(os.getgid()).gr_name
    return current_user, current_group


def get_owner(file_inp):
    """
    Retrieves the owner of the specified file.

    This function uses the `os` module to get the owner of the specified file.

    Parameters
    ----------
    file_inp : str or Path
        The path to the file whose owner is to be retrieved.

    Returns
    -------
    tuple
        A tuple containing the username and group name of the file's owner.
    """
    # Get the UID and GID of the file
    uid = os.stat(file_inp).st_uid
    gid = os.stat(file_inp).st_gid

    # Get the username and group name
    try:
        user = pwd.getpwuid(uid).pw_name
        group = grp.getgrgid(gid).gr_name
    except Exception as e:
        logger.warning(
            f"Unable to get name for owner/grp ID ({uid}:{gid}) of {os.path.basename(file_inp)}: {e}"
        )
        user, group = uid, gid

    return user, group


def change_owner(file_inp, user, group):
    """
    Changes the ownership of the specified file to the given user and group.

    Parameters
    ----------
    file_inp : str or Path
        The path to the file whose ownership is to be changed.
    user : str
        The username of the new owner.
    group : str
        The group name of the new owner.

    Returns
    -------
    None
    """
    user_ini, group_ini = get_owner(file_inp)

    if user_ini == user and group_ini == group:
        # Ownership of file_inp is already user:group
        pass
    else:
        # Get the UID and GID of the new owner/group
        uid = pwd.getpwnam(user).pw_uid
        gid = grp.getgrnam(group).gr_gid

        # Change the ownership
        try:
            os.chown(file_inp, uid, gid)
        except Exception as e:
            logger.warning(
                f"Unable to change owner {user_ini}:{group_ini} > {user}:{group}: {e}"
            )
    return None


def prep_rgx_custom(conv_regex_custom_main_inp=None, conv_regex_custom_annex_inp=None):
    """
    Prepares a custom regex function to catch converted files based on provided main and annex regex patterns.
    This function checks if both main and annex custom regex patterns are provided.
    If both are provided, it returns a function that always returns the tuple of these patterns.
    If only one or neither is
    provided, it logs a warning and returns None.

    Parameters
    ----------
    conv_regex_custom_main_inp : str or None
        Custom regex pattern for the main converted file.
    conv_regex_custom_annex_inp : str or None
        Custom regex pattern for the annex converted file.

    Returns
    -------
    function or None
        A function that returns a tuple of the provided regex patterns if both are provided,
        otherwise None.

    """
    if conv_regex_custom_main_inp and conv_regex_custom_annex_inp:
        logger.info(
            "main&annex custom regex for converted file provided: %s / %s",
            conv_regex_custom_main_inp,
            conv_regex_custom_annex_inp,
        )
        conv_regex_custom_tup = (conv_regex_custom_main_inp, conv_regex_custom_annex_inp)
        # here, conv_regex_fct_use, is the *function* itself (not the fonction return), then a lambda
        conv_regex_fct_out = lambda x: arocnv.conv_regex_custom(conv_regex_custom_tup)
    elif not conv_regex_custom_main_inp and not conv_regex_custom_annex_inp:
        logger.warning(
            "Error: both main&annex custom regex for converted file must be provided: %s & %s",
            conv_regex_custom_main_inp,
            conv_regex_custom_annex_inp,
        )
        logger.warning("No custom regex will be used.")
        conv_regex_fct_out = None
    else:
        conv_regex_fct_out = None

    return conv_regex_fct_out



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Apr  7 12:07:18 2023

@author: psakic
"""

from pathlib import Path

import numpy as np
import pandas as pd

import autorino.common as arocmn
import autorino.convert as arocnv
from geodezyx import operational as gzyx_opera

#### Import the logger
import logging
import autorino.cfgenv.env_read as aroenv

logger = logging.getLogger("autorino")
logger.setLevel(aroenv.ARO_ENV_DIC["general"]["log_level"])

BOLD_SRT = "\033[1m"
BOLD_END = "\033[0m"


class ConvertGnss(arocmn.StepGnss):
    """
    A class used to represent the GNSS conversion process.

    This class inherits from the StepGnss class
    and is used to handle the conversion of GNSS data.

    Attributes
    ----------
    out_dir : str
        The directory where the output of the conversion should be stored.
    tmp_dir : str
        The directory where temporary files during the conversion process should be stored.
    log_dir : str
        The directory where log files should be stored.
    epoch_range : tuple, optional
        A tuple containing the start and end epochs for the conversion process.
    site : str, optional
        The site for which the conversion process should be performed.
    session : str, optional
        The session for which the conversion process should be performed.
    options : dict, optional
        A dictionary containing any additional options for the conversion process.
    metadata : str or list, optional
        The metadata to be included in the converted RINEX files
        Possible inputs are:
        * list of string (sitelog file paths),
        * single string (single sitelog file path)
        * single string (directory containing the sitelogs)
        * list of MetaData objects
        * single MetaData object

    Methods
    -------
    __init__(self, out_dir, tmp_dir, log_dir, epoch_range_inp=None, site=None, session=None, options=None, metadata=None)
        Initializes the ConvertGnss class with the specified parameters.
    """

    def __init__(
        self,
        out_dir,
        tmp_dir,
        log_dir,
        inp_dir=None,
        inp_file_regex=None,
        epoch_range=None,
        site=None,
        session=None,
        options=None,
        metadata=None,
    ):
        """
        Initializes the ConvertGnss class with the specified parameters.

        Parameters
        ----------
        out_dir : str
            The directory where the output of the conversion should be stored.
        tmp_dir : str
            The directory where temporary files during the conversion process should be stored.
        log_dir : str
            The directory where log files should be stored.
        epoch_range : tuple, optional
            A tuple containing the start and end epochs for the conversion process.
        site : str, optional
            The site for which the conversion process should be performed.
        session : str, optional
            The session for which the conversion process should be performed.
        options : dict, optional
            A dictionary containing any additional options for the conversion process.
        metadata : str or list, optional
            The metadata to be included in the converted RINEX files
            Possible inputs are:
            * list of string (sitelog file paths),
            * single string (single sitelog file path)
            * single string (directory containing the sitelogs)
            * list of MetaData objects
            * single MetaData object
        """
        super().__init__(
            out_dir=out_dir,
            tmp_dir=tmp_dir,
            log_dir=log_dir,
            inp_dir=inp_dir,
            inp_file_regex=inp_file_regex,
            epoch_range=epoch_range,
            site=site,
            session=session,
            options=options,
            metadata=metadata,
        )

    ###############################################

    def convert(
        self,
        verbose=False,
        force=False,
        rinexmod_options=None,
        converter="auto",
        filter_prev_tables=False,
        conv_regex_custom_main=None,
        conv_regex_custom_annex=None,
    ):
        """
        "total action" method

        Executes the total conversion process for GNSS data.

        This method handles the entire conversion process for GNSS data.
        It sets up temporary directories, guesses and deactivates existing local RINEX files, decompresses files,
        filters and converts input files, applies rinexmod options, and finally moves the converted files to
        the final directory.
        It also handles logging and error checking throughout the process.

        Parameters
        ----------
        verbose : bool, optional
            If True, prints the conversion table. Default is False.
        force : bool, optional
            If True, forces the conversion even if output files already exist. Default is False.
        rinexmod_options : dict, optional
            A dictionary containing options for the rinexmod process.
             If not specified, default options are used.
        converter : str, optional
            The converter to be used for the conversion.
            If not specified, the best converter is automatically selected.
            Default is 'auto'.
        filter_prev_tables : bool, optional
            If True, filters and skip previously converted files
            with tables stored in the tmp tables directory.
            Default is False.
        conv_regex_custom_main : str, optional
            A custom regular expression to catch the main converted file.
            If not specified, no custom regex is used.
            Default is None.
        conv_regex_custom_annex : str, optional
            A custom regular expression to catch naming the annex converted file.
            If not specified, no custom regex is used.
            Default is None.

        Returns
        -------
        None
        """

        # #### Legacy Option
        # update_site_id_with_metadata : bool, optional
        #     If True, updates the site identifier using the metadata (default case).
        #     Since the site code from RAW file name can be poorly formatted
        #     we search it w.r.t. the sites from the metadata.
        #     If False, the site identifier is not updated, and it is the one of the
        #     ConvertGnss object self.site_id that is used.
        #     (for some advanced cases).
        #     Default is True.

        self.set_logfile()
        logger.info(BOLD_SRT + ">>>>>> RAW > RINEX files conversion" + BOLD_END)

        self.set_tmp_dirs()
        self.clean_tmp_dirs()
        # other tmps subdirs come also later in the loop
        self.set_translate_dict()
        # others translate dict updates will come in the loop

        if self.metadata:
            site4_list, site9_list = arocnv.site_list_from_metadata(self.metadata)
        else:
            site4_list, site9_list = [], []

        ### initialize the table as log
        self.set_table_log(out_dir=self.tmp_dir_tables)

        ### guess and deactivate existing local RINEX files
        # generate the potential local files
        self.guess_local_rnx()
        # tests if the input local files are here
        self.check_local_files("inp")
        # tests if the output local files are already there
        self.check_local_files("out")
        # be sure ok_xxx columns are booleans
        self.table_ok_cols_bool()
        # switch ok_inp to False if the output files are already there
        self.filter_ok_out()

        if force:
            self.force("convert")

        if filter_prev_tables:
            logger.debug(f"Loading filter previous tables in: {self.tmp_dir_tables:}")
            prv_tbl_df = arocmn.load_previous_tables(self.tmp_dir_tables)
            # Filter previous tables stored in log_dir
            if len(prv_tbl_df) > 0:
                self.get_vals_prev_tab(prv_tbl_df)
                self.filter_prev_tab(prv_tbl_df)
            # switch ok_inp to False if the output files are already there
            self.filter_ok_out()

        self.tmp_decmp_files, _ = self.decompress()

        # get a table with only the good files (ok_inp == True)
        # table_init_ok must be used only for the following statistics!
        self.table_ok_cols_bool()
        table_init_ok = self.filter_purge()

        n_tot_inp = len(self.table["ok_inp"])
        n_ok_inp = (self.table["ok_inp"]).sum()
        n_not_ok_inp = np.logical_not(self.table["ok_inp"]).sum()

        logger.info(
            "%5i/%5i files will be converted, %5i/%5i files are excluded",
            n_ok_inp,
            n_tot_inp,
            n_not_ok_inp,
            n_tot_inp,
        )

        if verbose:
            self.print_table()

        ######################### START THE LOOP ##############################
        for irow, row in self.table.iterrows():
            fraw = Path(self.table.loc[irow, "fpath_inp"])
            ext = fraw.suffix.lower()

            if not self.mono_ok_check(irow, "conversion"):
                continue

            logger.info(">>>> input raw file for conversion: %s", fraw.name)

            ###########################################################################
            # change the site_id here is a very bad idea, it f*cks the outdir 240605
            # (the outdir has not the country code anymore)
            #
            # but, because of the new IGS update (9 char in sitlog)
            # it should not be a pb anymore

            # +++ since the site code from fraw can be poorly formatted
            # we search it w.r.t. the sites from the metadata
            # we update the table row and the translate_dic (necessary for the output dir)
            self.mono_site_upd(irow, site4_list)
            # set self.site_id for the output dir translation & rinexmod options
            self.site_id = self.table.loc[irow, "site"]

            self.set_translate_dict()
            ###########################################################################
            # +++ CONVERTER SELECTION

            if converter != "auto":
                converter_name_use = converter  # converter is forced
            else:
                # ++ do a first converter selection by identifying odd files
                converter_name_use = arocnv.slct_conv_odd_f(fraw)
                # NB: converter selection for regular files is done in
                # autorino.conv_cmd_run._convert_select

            logger.info("extension/converter: %s/%s", ext, converter_name_use)

            if not converter_name_use:
                logger.info("file skipped, no converter found: %s", fraw)
                self.table.loc[irow, "note"] = "no converter found"
                self.table.loc[irow, "ok_inp"] = False
                self.write_in_table_log(self.table.loc[irow])

            ## prepare the custom regex function if any
            # if not, conv_regex_fct_use is None and the default regexs
            # from autorino.convert.converter_run are set later
            conv_regex_fct_use = arocnv.prep_rgx_custom(conv_regex_custom_main, conv_regex_custom_annex)

            # ++ a function to stop the docker containers running for too long
            # (for trimble conversion)
            arocnv.stop_old_docker()

            #############################################################
            # +++++ CONVERSION
            frnxtmp = self.mono_convert(
                irow, self.tmp_dir_converted,
                converter_inp=converter_name_use,
                conv_regex_fct_inp=conv_regex_fct_use
            )
            self.tmp_rnx_files.append(frnxtmp)  # list for final remove

            #############################################################
            # +++++ RINEXMOD
            rinexmod_options_use = self.updt_rnxmodopts(
                rinexmod_options, irow, debug_print=False
            )

            self.mono_rinexmod(
                irow, self.tmp_dir_rinexmoded,
                rinexmod_options=rinexmod_options_use
            )
            #############################################################

            # +++++ FINAL MOVE
            self.mono_mv_final(irow, force=force)

        # ++++ remove temporary files
        self.remov_tmp_files()

        # close the log file
        self.close_logfile()

        return None

    #               _   _
    #     /\       | | (_)
    #    /  \   ___| |_ _  ___  _ __  ___    ___  _ __    _ __ _____      _____
    #   / /\ \ / __| __| |/ _ \| '_ \/ __|  / _ \| '_ \  | '__/ _ \ \ /\ / / __|
    #  / ____ \ (__| |_| | (_) | | | \__ \ | (_) | | | | | | | (_) \ V  V /\__ \
    # /_/    \_\___|\__|_|\___/|_| |_|___/  \___/|_| |_| |_|  \___/ \_/\_/ |___/
    #

    def mono_convert(
        self, irow, out_dir=None, converter_inp="auto", table_col="fpath_inp", conv_regex_fct_inp=None
    ):
        """
        "on row" method

        Converts the 'table_col' entry for each row of the table.

        This method is applied to each row of the table. It converts the 'table_col' entry, typically a file.
        The conversion is performed using the specified converter.
        If no converter is specified, an automatic converter is used.
        The output of the conversion is stored in the specified output directory.
        If no output directory is specified, the output is stored in the temporary directory.

        Parameters
        ----------
        irow : int
            The index of the row in the table to be converted.
        out_dir : str, optional
            The directory where the output of the conversion should be stored.
             If not specified, the output is stored
            in the temporary directory.
        converter_inp : str, optional
            The converter to be used for the conversion.
             If not specified, an automatic converter is used.
        table_col : str, optional
            The column in the table that should be converted.
            Typically, this is a file. Default is 'fpath_inp'.
        conv_regex_fct_inp : function, optional
            A custom function returning regexs to catch
            the main and annex converted file names.

        Returns
        -------
        str
            The path of the converted file.
        """

        if not self.mono_ok_check(irow, "conversion"):
            return None

        # definition of the output directory (after the action)
        if out_dir:
            out_dir_use = out_dir
        elif hasattr(self, "tmp_dir_converted"):
            out_dir_use = self.tmp_dir_converted
        else:
            out_dir_use = self.tmp_dir

        try:
            frnxtmp, _ = arocnv.converter_run(
                self.table.loc[irow, table_col],
                out_dir_use,
                converter=converter_inp,
                conv_regex_fct= conv_regex_fct_inp
            )
        except Exception as e:
            logger.error("Error for: %s", self.table.loc[irow, table_col])
            logger.error("Exception raised: %s", e)
            frnxtmp = None

        if frnxtmp:
            ### update table if things go well
            self.table.loc[irow, "ok_out"] = True
            self.table.loc[irow, "fpath_out"] = frnxtmp
            epo_srt_ok, epo_end_ok = gzyx_opera.rinex_start_end(frnxtmp)
            self.table.loc[irow, "epoch_srt"] = pd.to_datetime(epo_srt_ok, utc=True)
            self.table.loc[irow, "epoch_end"] = pd.to_datetime(epo_end_ok, utc=True)
        else:
            ### update table if things go wrong
            self.table.loc[irow, "ok_out"] = False
        return frnxtmp

    def mono_site_upd(self, irow, metadata_or_sites_list_inp, force=False):
        """
        Updates the 'site' entry for each row of the table.

        This method is applied to each row of the table.
        It checks if the 'site' entry is defined.
        If it is not defined or if the 'force' parameter
        is set to True, it updates the 'site' entry.
        The update is performed by searching the site
        from a list of sites or metadata.
        If the 'site' entry is already defined and 'force' is not set to True,
        it sets the 'site' entry to 'XXXX00XXX'.

        Parameters
        ----------
        irow : int
            The index of the row in the table to be updated.
        metadata_or_sites_list_inp : list
            A list of sites or metadata from which the site should be searched.
        force : bool, optional
            If True, forces the update of the 'site'
            entry even if it is already defined.
            Default is False.

        Returns
        -------
        str
            The updated 'site' entry.
        """
        val_def = arocmn.is_ok(self.table.loc[irow, "site"])
        if not val_def or force:
            fraw = Path(self.table.loc[irow, "fpath_inp"])
            site_found = arocnv.site_search_from_list(fraw, metadata_or_sites_list_inp)
            self.table.loc[irow, "site"] = site_found
        else:
            site_found = "XXXX00XXX"

        return site_found



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Dec  1 15:47:05 2022

@author: psakic
"""

import collections.abc

# Create a logger object.
import os
import yaml
import mergedeep
from pprint import pprint

import autorino.common as arocmn
import autorino.convert as arocnv
import autorino.download as arodwl
import autorino.handle as arohdl

#### new rinexmod v4 import
import rinexmod.api as rimo_api
import rinexmod.classes as rimo_cls
#### old rinexmod import (for compatibility with older versions)
#from rinexmod import rinexmod_api as rimo_api
#from rinexmod import metadata as rimo_mda

import datetime as dt

#### Import the logger
import logging
import autorino.cfgenv.env_read as aroenv

logger = logging.getLogger("autorino")
logger.setLevel(aroenv.ARO_ENV_DIC["general"]["log_level"])

BOLD_SRT = "\033[1m"
BOLD_END = "\033[0m"


def load_cfg(cfg_path, verbose=True):
    """
    Loads quickly one or several configuration files (YAML format) **without interpretation**.

    This function reads a YAML configuration file from the specified path and returns the parsed content.

    Parameters
    ----------
    cfg_path : str or list of str
        The path(s) to the configuration file.
        If a list is provided, the config files are merged
    verbose : bool

    Returns
    -------
    dict
        The parsed/merged content of the configuration files.
    """

    recursive = True if isinstance(cfg_path, list) else False

    if verbose and not recursive:
        logger.info(BOLD_SRT + ">>>>>>>> Read configfile: " + BOLD_END + "%s", cfg_path)
    if recursive:
        ys_raw = [y for y in [load_cfg(c, verbose=verbose) for c in cfg_path] if y]
        return mergedeep.merge({}, *ys_raw)
    elif not cfg_path or not os.path.isfile(cfg_path):
        msg = f"config file doesn't exists!: {cfg_path}"
        logger.error(msg)
        raise FileNotFoundError(None, msg)
    else:
        y_out = yaml.safe_load(open(cfg_path))
        return y_out


def read_cfg(
    site_cfg_path, epoch_range=None, include_cfg_paths_xtra=None, verbose=False
):
    """
    Read and interpret a configuration file (YAML format) and
    return a list of StepGnss objects to be launched sequentially.

    This function takes in a path to a configuration file,
    an optional EpochRange object, and an optional path
    to a main configuration file.
    It reads the configuration file, updates it with the main configuration file if provided,
    and creates a list of StepGnss objects based on the configuration.
    The EpochRange object, if provided, will override the epoch ranges given in the configuration file.

    Parameters
    ----------
    site_cfg_path : str
        The path to the configuration file.
        Should be a single file here (no list).
    epoch_range : EpochRange, optional
        An EpochRange object which will override the epoch ranges
        given in the configuration file. Default is None.
    include_cfg_paths_xtra : str or list of str, optional
        The path to the include configuration file.
        If a list is provided, the include config files are merged.
        It overrides the files given with the 'include' keyword
        in the site configuration file.
        Default is None.
    verbose : bool, optional
        A flag indicating whether to print the configuration

    Returns
    -------
    steps_lis_lis : list
        A list of lists of StepGnss objects to be launched sequentially.
    steps_dic_dic : dict
        A dictionary of dictionaries of StepGnss.
    y_use : dict
        The parsed/merged content of the configuration file as dictionary.
    """

    y_site = load_cfg(site_cfg_path)

    if "include" in y_site.keys():
        include_cfg_paths = get_incl_cfg_paths(y_site, site_cfg_path)
        y_incl = load_cfg(include_cfg_paths, verbose=False)
    else:
        y_incl = load_cfg(include_cfg_paths_xtra, verbose=False)

    # before config files v20 (2025-04-10), the keyword FROM_MAIN was used in the sites cfgfiles
    # after v20, the main cfgfile is the basis, and is automatically overloaded with site cfgfile values
    legacy_main_updt = True if float(y_site["cfgfile_version"]) < 20.0 else False
    if legacy_main_updt:  # cfgfile_version < 20.
        y_use = read_cfg_legacy(y_site, y_incl)
    else:  # cfgfile_version >= 20.
        y_use = mergedeep.merge({}, y_incl, y_site) if y_incl else y_site.copy()

    if verbose:
        logger.debug(f"Used configuration (updated with include):\n{yaml.dump(y_use)}")

    steps_lis_lis, steps_dic_dic = read_cfg_core(y_use, epoch_range_inp=epoch_range)

    return steps_lis_lis, steps_dic_dic, y_use


def read_cfg_core(y_inp, epoch_range_inp=None):
    """
    Reads and interprets session configurations from a dictionary
    and returns lists and dictionaries of StepGnss objects.

    This function processes session configurations,
    loads metadata if available, and constructs lists and dictionaries
    of StepGnss objects based on the provided session configurations.

    Parameters
    ----------
    y_inp : dict
        A dictionary a site configuration.
    epoch_range_inp : EpochRange, optional
        An EpochRange object which will override the epoch ranges given
        in the session configurations. Default is None.

    Returns
    -------
    steps_lis_lis : list
        A list of lists of StepGnss objects to be launched sequentially.
    steps_dic_dic : dict
        A dictionary of dictionaries of StepGnss objects.
    """

    # ++++ METADATA
    y_station = y_inp["station"]
    y_sessions = y_station["sessions"]
    # Load metadata as external sitelog(s)
    if y_station["device"]["attributes_from_sitelog"]:
        slpath = y_station["site"]["sitelog_path"]
        if os.path.isdir(slpath) or os.path.isfile(slpath):
            # Load the metadata if the path is a directory or a file
            metadata = rimo_api.metadata_input_manage(slpath, force=False)
        else:
            # If not, keep the path it as a string
            # (because it might contain aliases and be translated later in the object)
            metadata = slpath
    # Load metadata as device block
    else:
        metadata = _device2mda(y_station)

    steps_lis_lis = []
    steps_dic_dic = {}
    for k_ses, y_ses in y_sessions.items():

        steps_lis = []
        steps_dic = {}

        y_gen = y_ses["general"]

        if not _is_cfg_bloc_active(y_gen):
            continue

        # y_gen_main = y_ses_main['general']
        # y_gen = update_w_main_dic(y_gen, y_gen_main)

        # ++++ TMP DIRECTORY
        tmp_dir, _, _ = _get_dir_path(y_gen, "tmp")

        # ++++ LOG DIRECTORY
        log_dir, _, _ = _get_dir_path(y_gen, "log")

        # ++++ EPOCH RANGE AT THE SESSION LEVEL
        if epoch_range_inp:
            epo_obj_ses = epoch_range_inp
        else:
            epo_obj_ses = _epoch_range_from_cfg_bloc(y_ses["epoch_range"])

        # ++++ manage steps
        y_steps = y_ses["steps"]
        # y_workflow_main = y_ses_main['workflow']

        for k_stp, y_stp in y_steps.items():
            # y_step_main = y_workflow_main[k_stp]
            # y_step = update_w_main_dic(y_step, y_step_main)

            if not _is_cfg_bloc_active(y_stp):
                continue

            # ++++ EPOCH RANGE AT THE STEP LEVEL
            step_cls = step_cls_select(k_stp)
            if not step_cls:
                logger.warning("unknown step %s, skip", k_stp)
                continue

            if (
                y_stp["epoch_range"] == "FROM_SESSION"
                or not "epoch_range" in y_stp.keys()
            ):
                epo_obj_stp = epo_obj_ses
                y_stp["epoch_range"] = y_ses["epoch_range"]
            else:
                epo_obj_stp = _epoch_range_from_cfg_bloc(y_stp["epoch_range"])

            ## concatenate the dir_parent and the structure
            # (but not the file_regex, it is just about the directory)
            out_dir, _, _ = _get_dir_path(y_stp, "out")
            inp_dir, _, _ = _get_dir_path(y_stp, "inp", check_parent_dir_exist=False)

            if "inp_file_regex" in y_stp.keys():
                inp_file_regex = y_stp["inp_file_regex"]
            else:
                logger.warning("Compatibility! inp_file_regex not defined in cfg files")
                logger.warning("upgrade your config file to >v15, set to .* for now")
                inp_file_regex = ".*"

            kwargs_for_step = {
                "out_dir": out_dir,
                "tmp_dir": tmp_dir,
                "log_dir": log_dir,
                "inp_dir": inp_dir,
                "inp_file_regex": inp_file_regex,
                "epoch_range": epo_obj_stp,
                "site": y_station["site"],
                "session": y_ses["general"],
                "options": y_stp["options"],
                "metadata": metadata,
            }

            if k_stp == "download":
                kwargs_for_step["access"] = y_station["access"]

            ### +++ CREATION OF THE OBJECT ###########
            step_obj = step_cls(**kwargs_for_step)
            ##########################################
            # appended in lis and dic at the end of the k_stp tests
            steps_lis.append(step_obj)
            steps_dic[k_stp] = step_obj

        steps_lis_lis.append(steps_lis)
        steps_dic_dic[k_ses] = steps_dic

    return steps_lis_lis, steps_dic_dic


def step_cls_select(step_name):
    """
    Selects the appropriate class for a given step name.

    This function maps a step name to its corresponding class, which is used
    to handle specific GNSS processing steps such as 'download', 'convert',
    'split', or 'splice'. If the step name is unknown, a warning is logged,
    and None is returned.

    Parameters
    ----------
    step_name : str
        The name of the step for which the corresponding class is to be selected.

    Returns
    -------
    class or None
        The class corresponding to the given step name, or None if the step name is unknown.

    Notes
    -----
    - Supported step names and their corresponding classes:
        - 'download': arodwl.DownloadGnss
        - 'convert': arocnv.ConvertGnss
        - 'split': arohdl.SplitGnss
        - 'splice': arohdl.SpliceGnss
    - Logs a warning if the step name is not recognized.
    """
    if step_name == "download":
        return arodwl.DownloadGnss
    elif step_name == "convert":
        return arocnv.ConvertGnss
    elif step_name == "split":
        return arohdl.SplitGnss
    elif step_name == "splice":
        return arohdl.SpliceGnss
    elif step_name in ("modify", "rinexmod"):
        if step_name == "rinexmod":
            warnmsg = "step 'rinexmod' is deprecated, use 'modify' instead"
            logger.warning(warnmsg)
            DeprecationWarning(warnmsg)
        return arohdl.ModifyGnss
    else:
        logger.warning("unknown step %s in cfgfiles file, skip", step_name)
        return None


def run_steps(
    steps_lis,
    steps_select_list=None,
    exclude_steps_select=False,
    verbose=True,
    force=False,
):
    """
    Executes the steps in the provided list.

    This function takes a list of StepGnss objects,
    an optional list of selected steps,
    and an optional boolean flag for printing tables.
    It iterates over the list of StepGnss objects and executes
    the 'download' or 'convert' method depending on the type of the step.
    If a list of selected steps is provided,
    only the steps in the list will be executed.
    If the 'verbose' flag is set to True,
    the tables will be printed during the execution of the steps.

    Parameters
    ----------
    steps_lis : Iterable
        A list of StepGnss objects to be executed.
    steps_select_list : list, optional
        A list of selected steps to be executed.
        If not provided, all steps in 'steps_lis' will be executed.
        Default is None.
    exclude_steps_select : bool, optional
        If True the selected steps indicated in step_select_list are excluded.
        It is the opposite behavior of the regular one using steps_list
        Default is False.
    verbose : bool, optional
        A flag indicating whether to print the tables during the execution of the steps.
         Default is True.
    force : bool, optional
        A flag indicating whether to force the execution of the steps.
        overrides the 'force' parameters in the configuration file.
        Default is False.

    Returns
    -------
    None
    """

    # If no steps are selected, initialize an empty list
    if not steps_select_list:
        steps_select_list = []

    wkf_prev = None

    # Log the number of steps to be run
    logger.info("%i steps will be run %s", len(steps_lis), steps_lis)

    # Iterate over the list of steps
    for istp, stp in enumerate(steps_lis):
        if istp > 0:
            wkf_prev = steps_lis[istp - 1]

        # Check if there are selected steps to be run
        if len(steps_select_list) > 0:
            # Forced case: steps_list contains the steps to be run only
            if (
                not exclude_steps_select
                and stp.get_step_type() not in steps_select_list
            ):
                logger.warning(
                    "step %s skipped, not selected in %s",
                    stp.get_step_type(),
                    steps_select_list,
                )
                continue
            # Exclusion case: steps_list contains steps to be excluded
            elif exclude_steps_select and stp.get_step_type() in steps_select_list:
                logger.warning(
                    "step %s skipped, selected in %s",
                    stp.get_step_type(),
                    steps_select_list,
                )
                continue
            else:
                pass

        # Set the verbose option if verbose is True
        if verbose:
            stp.options["verbose"] = True

        if force:
            stp.options["force"] = True

        load_table_msg_str = BOLD_SRT + ">>>>>>>> Load table for step %s" + BOLD_END
        # Execute the step based on its type
        if stp.get_step_type() == "download":
            stp.download(**stp.options)
        elif stp.get_step_type() == "convert":
            logger.info(load_table_msg_str, stp.get_step_type())
            stp.load_tab_inpdir()
            stp.convert(**stp.options)
        elif stp.get_step_type() == "splice":
            stp_rnx_inp = stp.copy()
            logger.info(load_table_msg_str, stp.get_step_type())
            stp_rnx_inp.load_tab_inpdir(update_epochs=True)
            stp.splice(input_mode="given", input_rinexs=stp_rnx_inp, **stp.options)
        elif stp.get_step_type() == "split":
            stp_rnx_inp = stp.copy()
            logger.info(load_table_msg_str, stp.get_step_type())
            stp_rnx_inp.load_tab_inpdir(update_epochs=True)
            stp.split(input_mode="given", input_rinexs=stp_rnx_inp, **stp.options)
        elif stp.get_step_type() in ("modify", "rinexmod"):
            if stp.get_step_type()  == "rinexmod":
                warnmsg = "step 'rinexmod' is deprecated, use 'modify' instead"
                logger.warning(warnmsg)
                DeprecationWarning(warnmsg)
            stp.load_tab_inpdir(update_epochs=True)
            stp.modify(**stp.options)

        ##### close the step

    return None


def _chk_parent_dir(parent_dir, parent_dir_key=None):
    """
    Checks if a parent directory exists and translates it with the environment variable first.

    This function takes a string representing a directory path.
    It first translates the directory path with the environment variable.
    Then, it checks if the translated directory exists.
    If it does not exist, it raises a FileNotFoundError with a custom error message.
    If it does exist, it returns None.

    This function is an internal function for the read_cfg function.

    Will translate it with the environment variable first.

    Parameters
    ----------
    parent_dir : str
        A string representing a directory path.
    parent_dir_key : str, optional
        The dictionnary key representing the parent directory.
        Default is None.

    Raises
    ------
    FileNotFoundError
        If the translated directory does not exist.

    Returns
    -------
    None
        If the translated directory exists.
    """
    parent_dir_out = arocmn.translator(parent_dir)

    mkdir_err_msg = " does not exists, create it manually first (mkdir -p ...)"

    ### FROM_MAIN is obsolete, this if test too
    # if parent_dir_out == "FROM_MAIN":
    #     # case when the parent directory is not defined in the main cfgfiles file
    #
    #     if not parent_dir_key:
    #         parent_dir_key = "a directory"
    #
    #     logger.error(
    #         "%s is not correctly defined in the main cfgfiles file "
    #         "(FROM_MAIN can not be replaced)",
    #         parent_dir_key,
    #     )
    #     raise FileNotFoundError(None, parent_dir_key + mkdir_err_msg)

    if not os.path.isdir(parent_dir_out):
        logger.error(parent_dir_out + mkdir_err_msg)
        raise FileNotFoundError(None, parent_dir_out + mkdir_err_msg)
    else:
        return None


def _is_cfg_bloc_active(ywkf):
    """
    Checks if a configuration block is active.

    This function takes a dictionary representing a configuration block.
    It checks if the 'active' key is present in the dictionary.
    If it is, it returns the value of the 'active' key.
    If the 'active' key is not present,
    it assumes the block is active and returns True.

    Internal function for read_cfg

    Parameters
    ----------
    ywkf : dict
        A dictionary representing a configuration block.

    Returns
    -------
    bool
        True if the 'active' key is present and its value is True,
        or if the 'active' key is not present.
        False if the 'active' key is present and its value is False.
    """
    if "active" in ywkf.keys():
        if ywkf["active"]:
            return True
        else:
            return False
    else:  # ++++ if no 'active' key, we assume the bloc active
        return True


def _epoch_range_from_cfg_bloc(epoch_range_dic):
    """
    get an EpochRange object from epoch_range dictionary bloc
    internal function for read_cfg
    """
    return arocmn.EpochRange(
        epoch_range_dic["epoch1"],
        epoch_range_dic["epoch2"],
        epoch_range_dic["period"],
        epoch_range_dic["round_method"],
        epoch_range_dic["tz"],
    )


def _get_dir_path(y_step, dir_type="out", check_parent_dir_exist=True):
    """
    Constructs a directory path based on the provided parameters.

    This function takes a dictionary containing step information,
     a directory type, and a flag indicating whether to check if
     the parent directory exists. It constructs a directory
     path by joining the parent directory and the structure.
     If the flag to check the existence of the parent directory
     is set to True, it checks if the parent directory exists.

    Parameters
    ----------
    y_step : dict
        A dictionary containing step information.
    dir_type : str, optional
        The type of directory to be constructed. Default is 'out'.
    check_parent_dir_exist : bool, optional
        A flag indicating whether to check if the parent directory exists.
        Default is True.

    Returns
    -------
    tuple
        A tuple containing the constructed directory path,
        the parent directory, and the structure.
    """

    def _key_check(key_inp, y_step_inp):
        if not key_inp in y_step_inp.keys():
            logger.error(f"no {key_parent} in {y_step}")
        return None

    key_parent = dir_type + "_dir_parent"
    key_structure = dir_type + "_dir_structure"

    _key_check(key_parent, y_step)
    dir_parent = y_step[key_parent]

    _key_check(key_structure, y_step)
    structure = y_step[key_structure]

    if check_parent_dir_exist:
        _chk_parent_dir(dir_parent, parent_dir_key=dir_type + "_dir_parent")

    dir_parent, structure = format_dir_path(dir_parent, structure)

    dir_path = os.path.join(dir_parent, structure)

    return dir_path, dir_parent, structure


def _device2mda(y_station):
    """
    Convert a device block from a configuration file to a MetaData object.
    """
    y_dev = y_station["device"]
    y_sit = y_station["site"]

    for attkw in ["rec_type", "rec_sn", "rec_fw", "ant_type", "ant_sn"]:
        if not attkw in y_dev.keys():
            logger.warning(
                "device attribute %s not found in cfg file, set to 'unknown'", attkw
            )
            y_dev[attkw] = "unknown"

    metadata = rimo_cls.MetaData()
    rec_dic = dict()
    rec_dic["Receiver Type"] = y_dev["rec_type"]
    rec_dic["Serial Number"] = str(y_dev["rec_sn"])
    rec_dic["Firmware Version"] = str(y_dev["rec_fw"])

    if not "rec_sat_sys" in y_dev.keys():
        rec_dic["Satellite System"] = "GPS+GLO+GAL+BDS+QZSS+SBAS"
    else:
        rec_dic["Satellite System"] = y_dev["rec_sat_sys"]

    ant_dic = dict()
    ant_dic["Antenna Type"] = y_dev["ant_type"]
    ant_dic["Antenna Radome Type"] = y_dev["ant_type"][-4:]

    ant_dic["Serial Number"] = y_dev["ant_sn"]

    ant_dic["Marker->ARP Up Ecc. (m)"] = y_dev["ecc_une"][0]
    ant_dic["Marker->ARP North Ecc(m)"] = y_dev["ecc_une"][1]
    ant_dic["Marker->ARP East Ecc(m)"] = y_dev["ecc_une"][2]

    metadata.add_instru(rec_dic, ant_dic)

    metadata.set_meta(
        site_id=y_sit["site_id"],
        domes=y_sit["domes"],
        operator=y_sit["operator"],
        x=y_sit["position_xyz"][0],
        y=y_sit["position_xyz"][1],
        z=y_sit["position_xyz"][2],
        country=y_sit["country"],
        date_prepared=dt.datetime.now(),
        agency=y_sit["agency"],
    )

    return metadata


def format_dir_path(dir_parent, structure):
    """
    Formats a directory path by adding or removing a leading slash.
    """

    if dir_parent[0] != "/":
        logger.warning(
            "dir_parent %s should start with slash (/), we add it automatically",
            dir_parent,
        )
        dir_parent = "/" + dir_parent

    if structure and structure[0] == "/":
        logger.warning(
            "structure %s should not start with slash (/), we remove it automatically",
            structure,
        )
        structure = structure[1:]

    return dir_parent, structure


def get_incl_cfg_paths(y_inp, site_cfg_path):
    """
    Generates a list of absolute paths for included configuration files.

    This function processes the 'include' section of a configuration dictionary,
    translates the paths using a translator function, and converts them to
    absolute paths relative to the provided site configuration file path.

    Parameters
    ----------
    y_inp : dict
        A dictionary containing the configuration data, including an 'include' key
        with paths to additional configuration files.
    site_cfg_path : str
        The absolute path to the site configuration file.

    Returns
    -------
    list
        A list of absolute paths for the included configuration files.
    """
    incl_cfg_paths_out = []
    for p in y_inp["include"]:
        # Translate the path using a custom translator function
        pmod = arocmn.translator(p)
        # Convert the path to be relative to the site configuration file's directory
        pmod = os.path.join(os.path.dirname(site_cfg_path), pmod)
        # Ensure the path is absolute with respect to the site configuration file
        pmod = arocmn.abs_path_wrt(pmod, site_cfg_path)
        incl_cfg_paths_out.append(pmod)
    return incl_cfg_paths_out


def get_incl_strategy(strat_str):
    """
    Retrieves the merging strategy for configuration files.

    This function maps a string representing a merging strategy to the corresponding
    `mergedeep.Strategy` enum value. Supported strategies are 'append' and 'replace'.

    Parameters
    ----------
    strat_str : str
        The merging strategy as a string. Valid values are 'append' or 'replace'.

    Returns
    -------
    mergedeep.Strategy
        The corresponding merging strategy.

    Raises
    ------
    ValueError
        If the provided strategy string is invalid.
    """
    if strat_str == "append":
        return mergedeep.Strategy.APPEND
    elif strat_str == "replace":
        return mergedeep.Strategy.REPLACE
    else:
        raise ValueError(
            f"Invalid strategy '{strat_str}'. "
            "Valid strategies are 'append' or 'replace'."
        )


#  _                                    __                  _   _
# | |                                  / _|                | | (_)
# | |     ___  __ _  __ _  ___ _   _  | |_ _   _ _ __   ___| |_ _  ___  _ __  ___
# | |    / _ \/ _` |/ _` |/ __| | | | |  _| | | | '_ \ / __| __| |/ _ \| '_ \/ __|
# | |___|  __/ (_| | (_| | (__| |_| | | | | |_| | | | | (__| |_| | (_) | | | \__ \
# |______\___|\__, |\__,_|\___|\__, | |_|  \__,_|_| |_|\___|\__|_|\___/|_| |_|___/
#              __/ |            __/ |
#             |___/            |___/


def check_from_main(y):
    """
    Check if the FROM_MAIN keyword is used in the configuration file.

    This function takes a dictionary representing the configuration file.
    It checks if the 'FROM_MAIN' keyword is used in the configuration file.
    If it is, it returns True. Otherwise, it returns False.

    Parameters
    ----------
    y : dict
        A dictionary representing the configuration file.

    Returns
    -------
    bool
        True if the 'FROM_MAIN' keyword is used in the configuration file.
        False otherwise.

    Warning
    -------
    Obsolete since 2025-04 and cfgfile v >= 20.0
    """
    if "FROM_MAIN" in str(y):
        return True
    else:
        return False


def update_w_main_dic(d, u=None, specific_value="FROM_MAIN"):
    """
    Updates a dictionary with another dictionary.

    Warning
    -------
    Obsolete since 2025-04 and cfgfile v >= 20.0
    """
    if u is None:
        return d
    for k, v in u.items():
        if k not in d.keys():
            continue
        if d[k] == specific_value:
            d[k] = v
        elif isinstance(v, collections.abc.Mapping):
            d[k] = update_w_main_dic(d.get(k, {}), v)
        else:
            if d[k] == specific_value:
                d[k] = v
    return d


def read_cfg_legacy(y_inp, y_main):
    """
    Handles legacy configuration files with the 'FROM_MAIN' keyword.

    This function processes configuration files with a version less than 20.0,
    where the 'FROM_MAIN' keyword is used to indicate that values should be
    inherited from a main configuration file. If the main configuration file
    is not provided or cannot be found, an error is raised.

    Parameters
    ----------
    y_inp : dict
        The input configuration dictionary (site-specific configuration).
    y_main : dict or None
        The main configuration dictionary. If None, the function will check
        for the presence of the 'FROM_MAIN' keyword in the input configuration.

    Returns
    -------
    dict
        The updated configuration dictionary, where values from the main
        configuration file have been merged into the input configuration.

    Raises
    ------
    FileNotFoundError
        If the 'FROM_MAIN' keyword is used in the input configuration but
        no main configuration file is provided.

    Warning
    -------
    Obsolete since 2025-04 and cfgfile v >= 20.0
    """
    # Log a warning indicating that a legacy configuration file is being used
    logger.warning("Use legacy cfgfile w/ FROM_MAIN keyword (cfgfile_version < 20.)")

    # Check if the main configuration file is missing and the 'FROM_MAIN' keyword is used
    if not y_main and check_from_main(y_inp):
        errmsg = "FROM_MAIN keyword used in cfg file, but no main cfg file provided (-m option)"
        logger.error(errmsg)  # Log an error message
        raise FileNotFoundError(
            None, errmsg
        )  # Raise an exception with the error message

    # Merge the input configuration with the main configuration
    y = update_w_main_dic(y_inp, y_main)

    # Return the updated configuration dictionary
    return y



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on 18/09/2024 18:25:28

@author: psakic
"""

import autorino.handle as arohdl
import autorino.common as arocmn

#### Import the logger
import logging
import autorino.cfgenv.env_read as aroenv

logger = logging.getLogger('autorino')
logger.setLevel(aroenv.ARO_ENV_DIC["general"]["log_level"])


def split_rnx(
    rnxs_inp,
    epoch_srt,
    epoch_end,
    period,
    out_dir,
    tmp_dir,
    log_dir=None,
    site=None,
    data_frequency="30S",
    handle_software="converto",
    rinexmod_options=None,
    metadata=None,
):
    """
    Frontend function to split RINEX files based on certain criteria

    Parameters
    ----------
    rnxs_inp : list
        The input RINEX files to be split
        The input can be:
        * a python list
        * a text file path containing a list of files
        * a tuple containing several text files path
        * a directory path.
    epoch_srt : datetime-like
        The start epoch for the splicing operation.
    epoch_end : datetime-like
        The end epoch for the splicing operation.
    period : str
        The period for the splicing operation.
    out_dir : str
        The output directory where the split files will be stored
    tmp_dir : str
        The temporary directory used during the splitting process
    log_dir : str, optional
        The directory where logs will be stored. If not provided, it defaults to tmp_dir
    site : str, optional
        The site name to be used for the split RINEX files
        Facultative but highly recommended to detect exisiting files to be skipped.
        data_frequency : str, optional
        The data frequency for the spliced RINEX files.
        Facultative but highly recommended to detect exisiting files to be skipped.
    handle_software : str, optional
        The software to be used for handling the RINEX files during the split operation
    rinexmod_options : dict, optional
        The options for modifying the RINEX files during the split operation
    metadata : str or list, optional
        The metadata to be included in the split RINEX files
        Possible inputs are:
         * list of string (sitelog file paths),
         * single string (single sitelog file path)
         * single string (directory containing the sitelogs)
         * list of MetaData objects
         * single MetaData object

    Returns
    -------
    None
    """
    if not log_dir:
        log_dir = tmp_dir

    epo_rng = arocmn.EpochRange(epoch_srt, epoch_end, period)

    spt = arohdl.SplitGnss(out_dir, tmp_dir, log_dir,
                           epoch_range=epo_rng,
                           site={'site_id':site},
                           session={"data_frequency": data_frequency},
                           metadata=metadata)

    spt.split(
        input_mode="given",
        input_rinexs=rnxs_inp,
        handle_software=handle_software,
        rinexmod_options=rinexmod_options,
        verbose=True
    )

    return spt




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Feb 21 16:23:06 2023

@author: psakic
"""

import datetime as dt
import os
import re
import subprocess
from pathlib import Path
from subprocess import PIPE
from typing import Union, List

import autorino.convert as arocnv
from geodezyx import utils, conv

#### Import the logger
import logging
import autorino.cfgenv.env_read as aroenv

logger = logging.getLogger("autorino")
logger.setLevel(aroenv.ARO_ENV_DIC["general"]["log_level"])

###################################################################
# +++++ conversion function


def _convert_select(converter_inp, inp_raw_fpath=None):
    """
    internal function for ``converter_run``.
    Find the correct RAW > RINEX converter and gives its corresponding attributes

    Returns directly those attributes based on `converter_inp` keyword,
    or can do a basic research based on the RAW file extension

    See also autorino.conv_fcts.slct_conv_odd_f
    for the converter selection of oddly named files

    Parameters
    ----------
    converter_inp : str
        name of the converter used.
        see ``converter_run`` help for more details
    inp_raw_fpath : Path, optional
        RAW file path. used for converter research based on the RAW
        file extension. The default is None.

    Returns
    -------
    converter_name : str
        converter's name.
    brand : str
        converter's name/manufacturer.
    cmd_build_fct : function
        interface function with the converter to perform the conversion.
        see ``converter_run``'s help for more details
    conv_regex_fct : function
        interface function to find the converted file with a regular expression.
    bin_options : list
        options for the conversion program. The default is [].
    bin_kwoptions : dict
        keyword options for the conversion program. The default is dict().
    """

    if converter_inp == "auto" and not inp_raw_fpath:
        logger.error(
            "not converter nor input file given, \
                  unable to returns the right conversion fcts"
        )
        raise Exception

    # + for RINEX handeling, inp_raw_fpath can ben an iterable (list)
    # + thus we just keep the 1st elt
    if utils.is_iterable(inp_raw_fpath):
        inp_raw_fpath = inp_raw_fpath[0]

    if converter_inp == "auto":
        inp_raw_fpath = Path(inp_raw_fpath)
        ext = inp_raw_fpath.suffix.upper()
        fname = inp_raw_fpath.name.upper()
        if ext in (".GZ", ".7Z", ".7ZIP", ".ZIP", ".Z"):
            logger.debug("%s is compressed", fname)
            ext = Path(Path(fname).stem).suffix.upper()
            fname = Path(Path(fname).stem).name.upper()

    else:
        inp_raw_fpath = Path(inp_raw_fpath)
        ext = ""
        fname = inp_raw_fpath.name.upper()

    # +++++ TRIMBLE
    # preliminary tests: Trimble default converter must be extracted from the environment values
    if ext in (".T00", ".T01", ".T02", ".T04") and converter_inp == "auto":
        converter_inp = aroenv.ARO_ENV_DIC["general"]["trimble_default_software"]
        logger.debug(
            "Trimble default converter defined in environnement: %s", converter_inp
        )

    # main test for Trimble : choose the right converter
    if ext in (".T00", ".T01", ".T02", ".T04") and converter_inp == "t0xconvert":
        converter_name = "t0xconvert"
        brand = "Trimble (official converter)"
        cmd_build_fct = arocnv.cmd_build_t0xconvert
        conv_regex_fct = arocnv.conv_regex_t0xconvert
        bin_options = []
        bin_kwoptions = dict()

    elif ext in (".T00", ".T01", ".T02", ".T04") and converter_inp == "trm2rinex":
        converter_name = "trm2rinex"
        brand = "Trimble (unofficial Docker converter)"
        cmd_build_fct = arocnv.cmd_build_trm2rinex
        conv_regex_fct = arocnv.conv_regex_trm2rinex
        bin_options = []
        bin_kwoptions = dict()

    elif ext == (".T00", ".T01", ".T02", ".T04") and converter_inp == "runpkr00":
        converter_name = "runpkr00"
        brand = "Trimble (legacy converter)"
        cmd_build_fct = arocnv.cmd_build_runpkr00
        conv_regex_fct = arocnv.conv_regex_runpkr00
        bin_options = []
        bin_kwoptions = dict()

    elif ext in (".TGD", "TG!") or converter_inp == "teqc":
        converter_name = "teqc"
        brand = "Trimble"
        cmd_build_fct = arocnv.cmd_build_teqc
        conv_regex_fct = arocnv.conv_regex_teqc
        bin_options = []
        bin_kwoptions = dict()

    # +++++ ASHTECH
    elif re.match(".([0-9]{3})", ext) and fname[0] in ("U", "R", "B"):
        converter_name = "teqc"
        brand = "Ashtech"
        cmd_build_fct = arocnv.cmd_build_teqc
        conv_regex_fct = arocnv.conv_regex_teqc
        yyyy, doy, week, dow, date = _ashtech_name_2_date(inp_raw_fpath)
        ftype = fname[0].lower()
        if ftype == "b":
            ftype = "d"
        bin_options = ["-ash " + ftype + " -week " + str(week)]
        # bin_kwoptions = {"-week": str(week)}
        bin_kwoptions = dict()

    # +++++ LEICA
    elif re.match(".(M[0-9]{2}|MDB)", ext) or converter_inp == "mdb2rinex":
        converter_name = "mdb2rinex"
        brand = "Leica"
        cmd_build_fct = arocnv.cmd_build_mdb2rinex
        conv_regex_fct = arocnv.conv_regex_mdb2rnx
        bin_options = []
        bin_kwoptions = dict()

    # +++++ SEPTENTRIO
    elif re.match(".([0-9]{2}_|.*A)", ext) or converter_inp == "sbf2rin":
        converter_name = "sbf2rin"
        brand = "Septentrio"
        cmd_build_fct = arocnv.cmd_build_sbf2rin
        conv_regex_fct = arocnv.conv_regex_void
        bin_options = []
        bin_kwoptions = dict()

    # +++++ GENERIC BINEX
    elif ext == ".BNX" or converter_inp == "convbin":
        converter_name = "convbin"
        brand = "Generic BINEX"
        cmd_build_fct = arocnv.cmd_build_convbin
        conv_regex_fct = arocnv.conv_regex_convbin
        bin_options = []
        bin_kwoptions = dict()

    # +++++ TOPCON
    elif ext == ".TPS" or converter_inp == "tps2rin":
        converter_name = "tps2rin"
        brand = "Topcon"
        cmd_build_fct = arocnv.cmd_build_tps2rin
        conv_regex_fct = arocnv.conv_regex_tps2rin
        bin_options = []
        bin_kwoptions = dict()

    # +++++ GFZRNX
    elif converter_inp == "gfzrnx":
        converter_name = "gfzrnx"
        brand = "RINEX Handeling (GFZ)"
        cmd_build_fct = arocnv.cmd_build_gfzrnx
        conv_regex_fct = arocnv.conv_regex_gfzrnx
        bin_options = []
        bin_kwoptions = dict()

    # +++++ CONVERTO
    elif converter_inp == "converto":
        converter_name = "converto"
        brand = "RINEX Handeling (IGN)"
        cmd_build_fct = arocnv.cmd_build_converto
        conv_regex_fct = arocnv.conv_regex_converto
        bin_options = []
        bin_kwoptions = dict()

    else:
        logger.error("unable to find the right converter for %s", inp_raw_fpath)
        logger.error(
            "input-given converter: %s, maybe not implemented yet?", converter_inp
        )
        raise Exception

    logger.debug("brand & converter selected: %s, %s", brand, converter_name)
    return (
        converter_name,
        brand,
        cmd_build_fct,
        conv_regex_fct,
        bin_options,
        bin_kwoptions,
    )


# set current user as constant
USER, GROUP = arocnv.get_current_user_grp()


def converter_run(
    inp_raw_fpath: Union[Path, str, List[Path], List[str]],
    out_dir: Union[Path, str],
    converter="auto",
    timeout=180,
    bin_options=[],
    bin_kwoptions=dict(),
    bin_path: Union[Path, str] = "",
    remove_converted_annex_files=True,
    cmd_build_fct=None,
    conv_regex_fct=None,
):
    """
    Generic function to run an external RAW > RINEX conversion program.
    It will detect automatically which converter has to be executed based on
    input RAW file extension,
    but this can be customized

    Parameters
    ----------
    inp_raw_fpath : Union[Path,str]
        path of the input RAW file.
        for RINEX Handeling (e.g. splice) a list of path is allowed.
    out_dir : Union[Path,str]
        destination directory of the converted RINEX.
    converter : str, optional
        name of the converter used.
        Supports :
            * 'auto' (automatic choice based on the extension),
            * 'trm2rnx' (Trimble unofficial),
            * 't0xconvert' (Trimble official),
            * 'runpkr00' (Trimble legacy),
            * 'teqc' (legacy conversion & RINEX Handeling),
            * 'mdb2rinex' (Leica),
            * 'sbf2rin' (Septentrio),
            * 'convbin' (BINEX),
            * 'tps2rin' (Topcon),
            * 'converto' (RINEX Handeling)
            * 'gfzrnx' (RINEX Handeling)
        see ``_convert_select`` function and ``cmd_build`` module
        for more details.
        The default is 'auto'.
    timeout : int, optional
        timeout in second for the conversion program call.
        The default is 60.
    bin_options : list, optional
        options for the conversion program. The default is [].
    bin_kwoptions : dict, optional
        keyword options for the conversion program. The default is dict().
    bin_path : Union[Path,str], optional
        path of the conversion program bin. The default is "".
    remove_converted_annex_files : bool, optional
        remove or not the 'annex' converted files.
        i.e. the not navigation files (e.g. the navigtation RINEXs).
        The default is True.
    cmd_build_fct : function, optional
        A custom function which build the command calling
        the conversion program
        See `cmd_build` module for more details.
        The default is None.
    conv_regex_fct : function, optional
        A custom function which build the regular expression to find
        the RINEX created during the conversion.
        See `cmd_regex` module for more details.
        The default is None.

    Raises
    ------
    FileNotFoundError

    Returns
    -------
    out_fpath
        the path of the converted RINEX.
    process_converter
        The subprocess object which ran the conversion (for debug purposes).
    """

    #### Convert the paths as Path objects
    out_dir = Path(out_dir)
    ## for RINEX handeling, inp_raw_fpath can ben an iterable (list)
    if utils.is_iterable(inp_raw_fpath):
        raw_fpath_multi = [Path(e) for e in inp_raw_fpath]
        raw_fpath_mono = raw_fpath_multi[0]
        raw_fpath = raw_fpath_multi
    else:  # a single file, most common case
        raw_fpath_multi = [Path(inp_raw_fpath)]
        raw_fpath_mono = Path(inp_raw_fpath)
        raw_fpath = raw_fpath_mono

    #### Check if input file exists
    for f in raw_fpath_multi:
        logger.debug("input file: %s", f)
        if not f.is_file():
            logger.error("input file not found: %s", f)
            raise FileNotFoundError

    if len(raw_fpath_multi) < 2:
        logger.info("input file for conversion: %s", raw_fpath_mono)
    else:
        logger.info("%i input file for conversion", len(raw_fpath_multi))

    # _convert_select can manage both a single file of a list*
    # then could handle both raw_fpath_mono or raw_fpath_multi
    # thus alias variable raw_fpath will work in both cases
    # *. but we thus we just keep the 1st list elt as the representent
    # of the full list (we assue it homogeneous)

    out_conv_sel = _convert_select(converter, raw_fpath)
    (
        converter_name,
        brand,
        cmd_build_fct_use,
        conv_regex_fct_use,
        bin_options_use,
        bin_kwoptions_use,
    ) = out_conv_sel

    #### Force the arocnv.cmd_build_fct, if any
    if cmd_build_fct:
        cmd_build_fct_use = cmd_build_fct

    #### Force the arocnv.conv_regex_fct, if any
    if conv_regex_fct:
        conv_regex_fct_use = conv_regex_fct

    #### Force the bin_options if any
    if bin_options:
        bin_options_use = bin_options

    #### Force the bin_kwoptions if any
    if bin_kwoptions:
        bin_kwoptions_use = bin_kwoptions

    #### build the command
    cmd_use, cmd_list, cmd_str = cmd_build_fct_use(
        raw_fpath, out_dir, bin_options_use, bin_kwoptions_use
    )
    ##### BIN PATH !!!!! XXXXX

    logger.debug("conversion command: %s", cmd_str)

    ############# run the external conversion programm #############
    timeout_reached = False
    start = dt.datetime.now()
    try:
        process_converter = subprocess.run(
            cmd_use,
            executable="/bin/bash",
            shell=True,
            stdout=PIPE,
            stderr=PIPE,
            timeout=timeout,
        )
    except subprocess.TimeoutExpired:
        process_converter = None
        timeout_reached = True

    end = dt.datetime.now()
    exec_time = (end - start).seconds + (end - start).microseconds * 10**-6

    #################################################################

    ###### check the output  on the conversion programm
    if timeout_reached:
        logger.error("Error while converting %s", raw_fpath_mono.name)
        logger.error("Timeout reached (%s seconds)", timeout)
    elif process_converter.returncode != 0:
        logger.error("Error while converting %s", raw_fpath_mono.name)
        logger.error("Converter's error message:")
        logger.error(process_converter.stderr.strip())
    else:
        logger.debug("Conversion done (%7.4f sec.). Converter's output:", exec_time)
        logger.debug(process_converter.stdout.strip())

    ###### get the converted file
    #### generate the regex matching the theoretical name for the converted file
    #### if a list of input file is given, the 1st one is used as output name
    conv_regex_main, conv_regex_annex = conv_regex_fct_use(raw_fpath_mono)
    logger.debug(
        "regex for the converted files (main/annex.): %s,%s",
        conv_regex_main,
        conv_regex_annex,
    )

    #### find the converted file matching the regex
    conv_files_main, conv_files_annex = find_conv_files(
        out_dir, conv_regex_main, conv_regex_annex
    )

    if not conv_files_main:
        out_fpath = ""
        logger.error("✗ converted file not found")
    else:
        out_fpath = Path(conv_files_main[0])
        logger.info(
            "✓ conversion OK (%7.4f sec.), main file/size: %s %s",
            exec_time,
            out_fpath,
            out_fpath.stat().st_size,
        )

    # change ownership
    if out_fpath:
        arocnv.change_owner(out_fpath, USER, GROUP)

    if remove_converted_annex_files:
        for f in conv_files_annex:
            os.remove(f)
            logger.debug("converted annex file removed: %s", f)

    return str(out_fpath), process_converter


#############################################################################
### Low level functions


def find_conv_files(directory, pattern_main, pattern_annex, n_sec=20):
    """
    Searches for the files in a directory that were recently created (within the last n_sec seconds)
    and match the main and annex patterns.

    This function iterates over all files in the specified directory and checks
    if they were created within the last n_sec seconds
    and if their names match the main or annex patterns.
    It returns two lists of files that match the main and annex patterns, respectively.

    Parameters
    ----------
    directory : Path
        The directory in which to search for files.
    pattern_main : str
        The regular expression pattern that the main files should match.
    pattern_annex : str
        The regular expression pattern that the annex files should match.
    n_sec : int, optional
        The number of seconds in the past to consider for file creation.
         Default is 20.

    Returns
    -------
    list
        The list of main files that were found.
    list
        The list of annex files that were found.
    """
    now = dt.datetime.now()
    delta = dt.timedelta(seconds=n_sec)
    files_main = []
    files_annex = []
    files_main_time = []
    files_annex_time = []
    for file in os.listdir(directory):
        filepath = os.path.join(directory, file)
        if os.path.isfile(filepath):
            created_time = dt.datetime.fromtimestamp(os.path.getctime(filepath))
            if now - created_time < delta and re.match(pattern_main, file):
                files_main.append(filepath)
                files_main_time.append(created_time)
            elif now - created_time < delta and re.match(pattern_annex, file):
                files_annex.append(filepath)
                files_annex_time.append(created_time)
            else:
                pass

    # Sort the files found
    files_main = [x for _, x in sorted(zip(files_main_time, files_main))]
    files_annex = [x for _, x in sorted(zip(files_annex_time, files_annex))]

    if len(files_main) > 1:
        logger.warning("Several converted main files found %s", files_main)
        files_main = [files_main[-1]]
        logger.warning("Keeping most recent only: %s", files_main[0])

    return files_main, files_annex


## https://stackoverflow.com/questions/36495669/difference-between-terms-option-argument-and-parameter
## https://tinf2.vub.ac.be/~dvermeir/mirrors/www-wks.acs.ohio-state.edu/unix_course/intro-14.html
## https://discourse.ubuntu.com/t/command-structure/18556


def _ashtech_name_2_date(inp_raw_fpath):
    """
    Extracts the record date from an ASHTECH file name.

    This function extracts the year, day of year, GPS week, and day of week from the name of an ASHTECH file.
    It also returns the date as a Python datetime object.

    Parameters
    ----------
    inp_raw_fpath : Path
        The path of the input ASHTECH file.

    Returns
    -------
    int
        The year extracted from the file name.
    int
        The day of the year extracted from the file name.
    int
        The GPS week extracted from the file name.
    int
        The day of the week extracted from the file name.
    datetime
        The date extracted from the file name as a Python datetime object.
    """

    inp_raw_fpath = Path(inp_raw_fpath)
    doy = int(inp_raw_fpath.suffix[1:])
    yy = int(inp_raw_fpath.stem[-2:])

    if yy < 80:
        y2k = 2000
    else:
        y2k = 1900

    yyyy = y2k + yy

    date = conv.doy2dt(yyyy, doy)
    week, dow = conv.dt2gpstime(date)

    return yyyy, doy, week, dow, date



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on 27/01/2025 09:35:53

@author: psakic
"""
import pandas as pd
import numpy as np

import autorino.handle as arohdl
import autorino.check as arochk
import rinexmod.classes as rimo_cls
import tqdm


class CheckGnss(arohdl.HandleGnss):
    def __init__(
        self,
        out_dir=None,
        tmp_dir=None,
        log_dir=None,
        inp_dir=None,
        inp_file_regex=None,
        epoch_range=None,
        site=None,
        session=None,
        options=None,
        metadata=None,
    ):

        super().__init__(
            out_dir=out_dir,
            tmp_dir=tmp_dir,
            log_dir=log_dir,
            inp_dir=inp_dir,
            inp_file_regex=inp_file_regex,
            epoch_range=epoch_range,
            site=site,
            session=session,
            options=options,
            metadata=metadata,
        )

        self.table_stats = pd.DataFrame()


    def analyze_rnxs(self):
        """
        this function do the basic analysis of the table of RINEXs

        Note
        ----
        Flags meaning

        * 0 = OK
        * 1 = missing RINEX or critical content
        * 2 = incomplete RINEX
        """

        self.table_stats = pd.DataFrame()

        ds_stk = []

        for irow, row in tqdm.tqdm(self.table.iterrows(), total=len(self.table),
                                   desc="Analyzing RINEX files for " + self.site_id):

            ds = dict()
            ds["fpath"] = self.table.loc[irow, "fpath_inp"]
            ds["site"] = self.table.loc[irow, "site"]

            if not self.mono_ok_check(int(irow), 'check'):
                ds["%"] = 0
            else:
                ### get RINEX as an rinexMod's Object
                rnxobj = rimo_cls.RinexFile(ds["fpath"])
                ### get RINEX site code
                ds["site"] = rnxobj.get_site(lower_case=False, only_4char=False)

                ### theoretical epochs
                ds["epoch_srt"] = self.table.loc[irow, "epoch_srt"]
                ds["epoch_end"] = self.table.loc[irow, "epoch_end"]

                ### get RINEX start/end in the data
                ds["epoch_srt_data"] = pd.to_datetime(rnxobj.start_date, format='%H:%M:%S')
                ds["epoch_end_data"] = pd.to_datetime(rnxobj.end_date, format='%H:%M:%S')
                ### get RINEX nominal interval
                ds["itrvl"] = rnxobj.sample_rate_numeric
                ### get RINEX number of epochs
                ds["nepochs"] = len(rnxobj.get_dates_all())
                ### get completness
                ds["td_str"] = rnxobj.get_file_period_from_filename()[0]

                # improve with right fct !!!!
                if ds["td_str"] == "01H":
                    ds["td_int"] = 3600
                elif ds["td_str"] == "01D":
                    ds["td_int"] = 86400
                else:
                    ds["td_int"] = np.nan

                ds["%"] = (ds["itrvl"] * ds["nepochs"] / ds["td_int"]) * 100
                ds["%"] = np.round(ds["%"], 0)

            ds_stk.append(ds)

        dfts = pd.DataFrame(ds_stk)
        self.table_stats = dfts

        return dfts


    def check(self):
        self.guess_local_rnx(io="inp")
        self.check_local_files(io="inp")
        self.print_table()
        self.analyze_rnxs()
        self.table["%"] = self.table_stats["%"]





#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on 18/09/2024 18:24:17

@author: psakic
"""

import os

import autorino.download as arodwl
import autorino.common as arocmn

#### Import the logger
import logging
import autorino.cfgenv.env_read as aroenv

logger = logging.getLogger("autorino")
logger.setLevel(aroenv.ARO_ENV_DIC["general"]["log_level"])


def download_raw(
    epoch_srt,
    epoch_end,
    period,
    hostname,
    inp_dir_parent,
    inp_dir_structure,
    inp_file_regex,
    out_dir_parent,
    out_structure="<SITE_ID9>/%Y/",
    site_id="XXXX00XXX",
    login="",
    password="",
    tmp_dir=None,
    log_dir=None,
    options=dict(),
    session=dict(),
):
    """
    Downloads raw GNSS data files.

    This function downloads raw GNSS data files for a specified epoch range and stores them
    in the specified output directory.

    Parameters
    ----------
    epoch_srt : datetime-like
        The start epoch for the splicing operation.
    epoch_end : datetime-like
        The end epoch for the splicing operation.
    period : str
        The period for the splicing operation.
    out_dir_parent : str
        The parent output directory where the downloaded files will be stored.
    out_structure : str
        The structure of the output sub-directory where the downloaded files will be stored.
        Typical values are '<SITE_ID4>/%Y/' or '%Y/%j/'.
        Default value is '<SITE_ID4>/%Y/'.
    hostname : str
        The hostname of the server from which the data files will be downloaded.
    inp_dir_parent : str
        The parent directory on the server where the raw data files are located.
    inp_dir_structure : str
        The raw file generic name structure on the server.
    inp_file_regex : str
        The regular expression used to match the raw data files on the server
    site_id : str, optional
        The site identifier for the data files. Default is "XXXX00XXX".
    login : str, optional
        The login username for the server. Default is an empty string.
    password : str, optional
        The login password for the server. Default is an empty string.
    tmp_dir : str, optional
        The temporary directory used during the download process. Default is None.
    log_dir : str, optional
        The directory where logs will be stored. If not provided, it defaults to tmp_dir. Default is None.
    options : dict, optional
        Additional options for the download process. Default is an empty dictionary.
    session : dict, optional
        Session information for the download process. Default is an empty dictionary.

    Returns
    -------
    object
        The DownloadGnss object after the download operation.
    """

    access_dic = dict()
    access_dic["hostname"] = hostname
    access_dic["login"] = login
    access_dic["password"] = password

    site_dic = dict()
    site_dic["site_id"] = site_id

    epoch_range = arocmn.EpochRange(epoch_srt, epoch_end, period)

    inp_dir = os.path.join(inp_dir_parent, inp_dir_structure)
    out_dir = os.path.join(out_dir_parent, out_structure)

    dwl = arodwl.DownloadGnss(
        out_dir=out_dir,
        tmp_dir=tmp_dir,
        log_dir=log_dir,
        inp_dir=inp_dir,
        inp_file_regex=inp_file_regex,
        epoch_range=epoch_range,
        access=access_dic,
        site=site_dic,
        session=session,
        options=options,
    )

    dwl.download()

    return dwl



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Mar 14 12:03:40 2023

@author: psakic

#### converted files regular expressions functions
#### main = the regex for the main file i.e. the Observation RINEX
#### annex = the regex for the ALL outputed files (Observation RINEX included)
#### the main with be processed before the annex,
#### thus annex regex will finally not include the main
"""

import re
from pathlib import Path


###################################################################


def conv_regex_void(f):
    """
    Generate the regular expressions of the main and annex converted files
    equivalent to the input filename

    ** Must be used for test purposes only !**

    It has the same behavior as all the `conv_regex` functions
    See note below

    Parameters
    ----------
    f : str or Path
        the input Raw filename or path
        (the filename will be extracted in the function).

    Returns
    -------
    conv_regex_main & conv_regex_annex : Complied Regex Objects
        The regular expressions.

    Note
    ----
    general behavior of the `conv_regex` functions:
    main = the regex for the main file i.e. the Observation RINEX
    annex = the regex for the ALL outputed files (Observation RINEX included)
    the main with be processed before the annex,
    thus annex regex will finally not include the main one
    """

    f = Path(Path(f).name)  ### keep the filename only
    conv_regex_main = re.compile(f.name)
    conv_regex_annex = re.compile(f.name)
    return conv_regex_main, conv_regex_annex


def conv_regex_custom(regex_tup_inp):
    """
    Generate the regular expressions of the main and annex converted files
    equivalent to the input regex tuple

    It has the same behavior as all the `conv_regex` functions
    See note below
    **but** the input is a tuple of regex strings
    not the input raw filename

    Parameters
    ----------
    regex_tup_inp : 2-tuple of str
        the input tuple of 2 regex strings for main and annex files

    Returns
    -------
    conv_regex_main & conv_regex_annex : Complied Regex Objects
        The regular expressions.

    Note
    ----
    general behavior of the `conv_regex` functions:
    main = the regex for the main file i.e. the Observation RINEX
    annex = the regex for the ALL outputed files (Observation RINEX included)
    the main with be processed before the annex,
    thus annex regex will finally not include the main one

    """
    regex_main, regex_annex = regex_tup_inp
    conv_regex_main = re.compile(regex_main)
    conv_regex_annex = re.compile(regex_annex)
    return conv_regex_main, conv_regex_annex


def conv_regex_runpkr00(f):
    """
    Generate the regular expressions of the mainand annex converted files
    outputed by runpkr00 (Trimble)

    It has the same behavior as all the `conv_regex` functions
    See note below

    Parameters
    ----------
    f : str or Path
        the input Raw filename or path
        (the filename will be extracted in the function).

    Returns
    -------
    conv_regex_main & conv_regex_annex : Complied Regex Objects
        The regular expressions.

    Note
    ----
    general behavior of the `conv_regex` functions:
    main = the regex for the main file i.e. the Observation RINEX
    annex = the regex for the ALL outputed files (Observation RINEX included)
    the main with be processed before the annex,
    thus annex regex will finally not include the main one
    """

    # PSA1______202110270000A.tgd (option -d -g)
    # PSA1______202110270000A.tg! (opened/working file)
    # ABD0______202110270000A.dat (when not option -g in runpkr)
    f = Path(Path(f).name)  ### keep the filename only
    conv_regex_main = re.compile(f.with_suffix(".tgd").name)
    conv_regex_annex = re.compile(f.stem)
    return conv_regex_main, conv_regex_annex


def conv_regex_teqc(f):
    """
    Generate the regular expressions of the main and annex converted files
    outputed by teqc (polyvalent)

    It has the same behavior as all the `conv_regex` functions
    See note below

    Parameters
    ----------
    f : str or Path
        the input Raw filename or path
        (the filename will be extracted in the function).

    Returns
    -------
    conv_regex_main & conv_regex_annex : Complied Regex Objects
        The regular expressions.

    Note
    ----
    general behavior of the `conv_regex` functions:
    main = the regex for the main file i.e. the Observation RINEX
    annex = the regex for the ALL outputed files (Observation RINEX included)
    the main with be processed before the annex,
    thus annex regex will finally not include the main one
    """
    f = Path(Path(f).name)  ### keep the filename only
    # conv_regex_main = re.compile(f.with_suffix(".rnx_teqc").name)
    conv_regex_main = re.compile(f.name + ".rnx_teqc")
    conv_regex_annex = conv_regex_main
    return conv_regex_main, conv_regex_annex


def conv_regex_converto(f):
    """
    Generate the regular expressions of the main and annex converted files
    outputed by converto (polyvalent)

    It has the same behavior as all the `conv_regex` functions
    See note below

    Parameters
    ----------
    f : str or Path
        the input Raw filename or path
        (the filename will be extracted in the function).

    Returns
    -------
    conv_regex_main & conv_regex_annex : Complied Regex Objects
        The regular expressions.

    Note
    ----
    general behavior of the `conv_regex` functions:
    main = the regex for the main file i.e. the Observation RINEX
    annex = the regex for the ALL outputed files (Observation RINEX included)
    the main with be processed before the annex,
    thus annex regex will finally not include the main one
    """
    f = Path(Path(f).name)  ### keep the filename only
    # conv_regex_main = re.compile(f.with_suffix(".rnx_teqc").name)
    conv_regex_main = re.compile(f.name + ".rnx_converto")
    conv_regex_annex = conv_regex_main
    return conv_regex_main, conv_regex_annex


def conv_regex_gfzrnx(f):
    """
    Generate the regular expressions of the main and annex converted files
    outputed by GFZRNX (polyvalent)

    It has the same behavior as all the `conv_regex` functions
    See note below

    Parameters
    ----------
    f : str or Path
        the input Raw filename or path
        (the filename will be extracted in the function).

    Returns
    -------
    conv_regex_main & conv_regex_annex : Complied Regex Objects
        The regular expressions.

    Note
    ----
    general behavior of the `conv_regex` functions:
    main = the regex for the main file i.e. the Observation RINEX
    annex = the regex for the ALL outputed files (Observation RINEX included)
    the main with be processed before the annex,
    thus annex regex will finally not include the main one
    """
    f = Path(Path(f).name)  ### keep the filename only
    # conv_regex_main = re.compile(f.with_suffix(".rnx_teqc").name)
    conv_regex_main = re.compile(f.name + ".rnx_gfzrnx")
    conv_regex_annex = conv_regex_main
    return conv_regex_main, conv_regex_annex


def conv_regex_trm2rinex(f):
    """
    Generate the regular expressions of the mainand annex converted files
    outputed by trm2rinex (Trimble)

    It has the same behavior as all the `conv_regex` functions
    See note below

    Parameters
    ----------
    f : str or Path
        the input Raw filename or path
        (the filename will be extracted in the function).

    Returns
    -------
    conv_regex_main & conv_regex_annex : Complied Regex Objects
        The regular expressions.

    Note
    ----
    general behavior of the `conv_regex` functions:
    main = the regex for the main file i.e. the Observation RINEX
    annex = the regex for the ALL outputed files (Observation RINEX included)
    the main with be processed before the annex,
    thus annex regex will finally not include the main one
    """

    # "/home/psakicki/aaa_FOURBI/convertertest/AGAL______202110270000A.21o",
    # "/home/psakicki/aaa_FOURBI/convertertest/AGAL______202110270000A.21l",
    # "/home/psakicki/aaa_FOURBI/convertertest/AGAL______202110270000A.21g",
    # "/home/psakicki/aaa_FOURBI/convertertest/AGAL______202110270000A.21n"
    f = Path(Path(f).name)  ### keep the filename only
    # date_full = re.search("[0-9]{12}[a-zA-Z]",f.name).group() ##too restrictive, sometime there is no final letter
    date_full = re.search("20[0-9]{10}", f.name).group()
    yyyy = date_full[:4]
    conv_regex_main = re.compile(f.with_suffix("." + yyyy[2:] + "o").name)
    conv_regex_annex = re.compile(f.with_suffix("." + yyyy[2:]).name)
    return conv_regex_main, conv_regex_annex


def conv_regex_t0xconvert(f):
    """
    Generate the regular expressions of the mainand annex converted files
    outputed by t0xconvert (Trimble)

    It has the same behavior as all the `conv_regex` functions
    See note below

    Parameters
    ----------
    f : str or Path
        the input Raw filename or path
        (the filename will be extracted in the function).

    Returns
    -------
    conv_regex_main & conv_regex_annex : Complied Regex Objects
        The regular expressions.

    Note
    ----
    general behavior of the `conv_regex` functions:
    main = the regex for the main file i.e. the Observation RINEX
    annex = the regex for the ALL outputed files (Observation RINEX included)
    the main with be processed before the annex,
    thus annex regex will finally not include the main one
    """

    f = Path(Path(f).name)  ### keep the filename only
    # date_full = re.search("[0-9]{12}[a-zA-Z]",f.name).group() ##too restrictive, sometime there is no final letter
    date_full = re.search("20[0-9]{10}", f.name).group()
    yyyy = date_full[:4]
    conv_regex_main = re.compile(f.with_suffix("." + yyyy[2:] + "o").name)
    conv_regex_annex = re.compile(f.with_suffix("." + yyyy[2:]).name)
    return conv_regex_main, conv_regex_annex


def conv_regex_mdb2rnx(f):
    """
    Generate the regular expressions of the mainand annex converted files
    outputed by mdb2rnx (Leica)

    It has the same behavior as all the `conv_regex` functions
    See note below

    Parameters
    ----------
    f : str or Path
        the input Raw filename or path
        (the filename will be extracted in the function).

    Returns
    -------
    conv_regex_main & conv_regex_annex : Complied Regex Objects
        The regular expressions.

    Note
    ----
    general behavior of the `conv_regex` functions:
    main = the regex for the main file i.e. the Observation RINEX
    annex = the regex for the ALL outputed files (Observation RINEX included)
    the main with be processed before the annex,
    thus annex regex will finally not include the main one
    """

    # souf3000.21o
    # souf3000.21n
    # souf3000.21l
    # souf3000.21g
    ### met also for OVSM
    # fsdc176p11.18o
    # mlm0236a00.18n
    # mlm0236a00.18l
    # mlm0236a00.18g
    # fsdc176p11.18o
    # fsdc176p11.18n
    # fsdc176p11.18g
    # and even for OVSG
    # souf226n 8.24l
    # souf226n 8.24n
    # souf226n 8.24o

    finp = str(f)
    f = Path(Path(f).name)  ### keep the filename only
    if finp.lower().endswith("mdb"):
        # used in OVSM e.g.
        # LAJB0a18002.MDB
        # LMLM0-Leicaa18233.MDB
        regex_doy_site = r".(\w{4})(-Leica)?.([0-9]{2})([0-9]{3})"
        doygroup = 4
        # alternativelywe can exclude the group 'Leica-' with ?:
        # regex_doy_site=r".(\w{4})(?:-Leica)?.([0-9]{2})([0-9]{3})"
        # doygroup = 3
    elif finp.lower().endswith("m00"):
        # used in OVSG e.g.
        # PAR1323a.m00
        regex_doy_site = r"(\w{4})([0-9]{3})"
        doygroup = 2
    else:
        regex_doy_site = r"(\w{4})([0-9]{3})"
        doygroup = 2

    site = re.match(regex_doy_site, f.name).group(1).lower()
    doy = re.match(regex_doy_site, f.name).group(doygroup).lower()
    conv_regex_main = re.compile(site + doy + r".(.{2})?\.[0-9]{2}o")
    conv_regex_annex = re.compile(site + doy + r".(.{2})?\.[0-9]{2}\w")
    return conv_regex_main, conv_regex_annex


def conv_regex_convbin(f):
    """
    Generate the regular expressions of the mainand annex converted files
    outputed by RTKLIB's convbin (polyvalent)

    It has the same behavior as all the `conv_regex` functions
    See note below

    Parameters
    ----------
    f : str or Path
        the input Raw filename or path
        (the filename will be extracted in the function).

    Returns
    -------
    conv_regex_main & conv_regex_annex : Complied Regex Objects
        The regular expressions.

    Note
    ----
    general behavior of the `conv_regex` functions:
    main = the regex for the main file i.e. the Observation RINEX
    annex = the regex for the ALL outputed files (Observation RINEX included)
    the main with be processed before the annex,
    thus annex regex will finally not include the main one
    """
    # Input
    # NBIM0a20221226.BNX
    # Output
    # NBIM0a20221226.obs
    # NBIM0a20221226.nav
    # NBIM0a20221226.sbs
    f = Path(Path(f).name)  ### keep the filename only
    conv_regex_main = re.compile(f.with_suffix(".obs").name)
    conv_regex_annex = re.compile(f.stem)
    return conv_regex_main, conv_regex_annex


def conv_regex_tps2rin(f):
    """
    Generate the regular expressions of the main and annex converted files
    outputed by tps2rin (Topcon)

    It has the same behavior as all the `conv_regex` functions
    See note below

    Parameters
    ----------
    f : str or Path
        the input Raw filename or path
        (the filename will be extracted in the function).

    Returns
    -------
    conv_regex_main & conv_regex_annex : Complied Regex Objects
        The regular expressions.

    Note
    ----
    general behavior of the `conv_regex` functions:
    main = the regex for the main file i.e. the Observation RINEX
    annex = the regex for the ALL outputed files (Observation RINEX included)
    the main with be processed before the annex,
    thus annex regex will finally not include the main one
    """
    # Input
    # DHS030011.TPS
    # Output
    # an extra character is added if there is already an existing file
    # dhs030002.11p
    # dhs030002.11p
    # dhs030002.11o
    # dhs030001.11p
    # dhs030001.11o
    # dhs030000.11p
    # dhs030000.11o
    # dhs03000.11p
    # dhs03000.11o

    f = Path(Path(f).name)  ### keep the filename only
    ### this regey_doy_site is too restricitive, it does not work for all tps files
    #regex_doy_site = r"(\w{4})([0-9]{3})"
    #site = re.match(regex_doy_site, f.name).group(1).lower()
    # a less restricitive regex_site
    regex_site = r"^(\w{4})"
    site = re.match(regex_site, f.name).group(1).lower()

    # doy=re.match(regex_doy_site,f.name).group(2).lower()
    # conv_regex_main = re.compile(site+doy+".(.|)\.[0-9]{2}o")
    # conv_regex_annex  = re.compile(site+doy+".(.|)\.[0-9]{2}\w")
    ## the date of the raw file can be actually anything...
    ## doy/month-day/etc..
    conv_regex_main = re.compile(site + r"[0-9]{3}.(.|)\.[0-9]{2}o")
    conv_regex_annex = re.compile(site + r"[0-9]{3}.(.|)\.[0-9]{2}\w")

    return conv_regex_main, conv_regex_annex




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on 18/09/2024 18:23:30

@author: psakic
"""

import glob
import os
import autorino.cfgfiles as arocfg
import autorino.common as arocmn

#### Import the logger
import logging
import autorino.cfgenv.env_read as aroenv

logger = logging.getLogger("autorino")
logger.setLevel(aroenv.ARO_ENV_DIC["general"]["log_level"])


def cfgfile_run(
    cfg_in,
    incl_cfg_in,
    sites_list=None,
    exclude_sites=False,
    epo_srt=None,
    epo_end=None,
    period="1D",
    steps_list=None,
    exclude_steps=False,
    force=False,
):
    """
    Run the Autorino configuration files.

    This function takes in a configuration file or a directory of configuration files,
    reads the configuration, and runs the steps specified in the configuration.

    Parameters
    ----------
    cfg_in : str
        The input configuration file or directory of configuration files.
        If a directory is provided, all files ending with '.yml' will be used.
    incl_cfg_in : str or list of str
        The include configuration files to be used for development/advanced purposes.
        If a list is provided, all files in the list will be included.
        It will override the `include` section of the `cfg_in` configuration file.
    sites_list : list, optional
        A list of site identifiers to filter the configuration files.
         If provided, only configurations for sites in this list will be processed.
         Default is None.
    exclude_sites : bool, optional
        If True, the site in sites_list will be ignored.
        It is the opposed behavior of the regular one using sites_list.
        Default is False.
    epo_srt : str, list, optional
        The start date for the epoch range.
        Can be a list; if so, each epoch is considered separately.
        Can be a file path; if so, the file contains a list of start epochs
        The epoch can be formatted as:
        * a litteral, e.g. 'yesterday', '10 days ago'
        * YYYY-DDD, year-day of year, e.g. 2025-140
        * YYYY-MM-DD, classic calendar date, e.g. 2025-05-20
        Default is None.
    epo_end : str, optional
        The end date for the epoch range. Default is None.
    period : str, optional
        The period for the epoch range. Default is "1D".
    steps_list : list, optional
        A list of selected steps to be executed.
        If not provided, all steps in 'steps_lis' will be executed.
        Default is None.
    exclude_steps : bool, optional
        If True the selected steps indicated in step_select_list are excluded.
        It is the opposite behavior of the regular one using steps_select_list
        Default is False.
    force : bool, optional
        If True, the steps will be executed even if the output files already exist.
        overrides the 'force' parameters in the configuration file.
        Default is False.

    Raises
    ------
    Exception
        If the provided cfg_in does not exist as a file or directory, an exception is raised.

    Returns
    -------
    exit_code_max : int
        The maximum exit code from all steps executed in the configuration files.
        If no steps were executed, returns 0.
    """

    # Check if cfg_in is a directory or a file and get the list of configuration files
    if os.path.isdir(cfg_in):
        cfg_use_lis = []
        for ext in ("/*yml", "/*yaml"):
            cfg_use_lis.extend(list(sorted(glob.glob(cfg_in + ext))))
    elif os.path.isfile(cfg_in):
        cfg_use_lis = [cfg_in]
    else:
        logger.error("%s does not exist, check input cfgfiles file/dir", cfg_in)
        return None

    # Determine the epoch range based on the provided start and end dates
    if epo_srt and epo_end:
        epoch_range = arocmn.EpochRange(epo_srt, epo_end, period)
    elif epo_srt and not epo_end:
        if os.path.isfile(epo_srt):
            with open(epo_srt, "r") as f:
                start_use = f.read().splitlines()
        elif isinstance(epo_srt, list):
            start_use = epo_srt
        else:
            logger.critical("start must be a list or a file path")
            return None
        epoch_range = arocmn.EpochRange(start_use, period=period)
    else:
        epoch_range = None

    # Process each configuration file
    for cfg_use in cfg_use_lis:
        if sites_list:
            # Quick load to check if the site is in the list or not
            y_quick = arocfg.load_cfg(cfg_path=cfg_use)
            site_quick = y_quick["station"]["site"]["site_id"]
            ### case 1: sites_list are the sites we want
            if not exclude_sites and (site_quick not in sites_list):
                logger.info("Skipping site %s (not in sites list)", site_quick)
                continue
            ### case 2: sites_list are the sites we ignore
            elif exclude_sites and (site_quick in sites_list):
                logger.info("Skipping site %s (in ignored sites list)", site_quick)
                continue
            ### case 3: regular case
            else:
                pass

        # Read the configuration and run the steps
        # step_lis_lis is a list of list because you can have several sessions in the same configuration file
        steps_lis_lis, steps_dic_dic, y_use = arocfg.read_cfg(
            site_cfg_path=cfg_use,
            include_cfg_paths_xtra=incl_cfg_in,
            epoch_range=epoch_range,
        )

        for steps_lis in steps_lis_lis:
            arocfg.run_steps(
                steps_lis,
                steps_select_list=steps_list,
                exclude_steps_select=exclude_steps,
                force=force,
            )

    # Get the maximum exit code from all steps
    exit_code_max = max(
        [max([stp.exit_code for stp in steps_lis]) for steps_lis in steps_lis_lis]
    )

    return exit_code_max



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on 22/04/2024 16:16:23

@author: psakic
"""
# Create a logger object.
import logging
import os
import collections.abc
import yaml
import mergedeep
import autorino
import rinexmod

### we need to clear the root logger to avoid duplicate logs
root_logger = logging.getLogger()
root_logger.handlers.clear()  # Clear all handlers in the root logger

logger = logging.getLogger("autorino")
logger.setLevel("DEBUG")


def read_env(envfile_path=None):
    """
    Reads an environment configuration file (YAML format) and returns the corresponding dictionary.

    Priority for determining the environment file path:
    1. Function argument
    2. Environment variable `AUTORINO_ENV`
    3. Default file in the current directory
    """

    varo = autorino.__version__
    vrimo = rinexmod.__version__
    logger.info("autorino & rinexmod version: %s & %s", varo, vrimo)

    # Default environment file path
    default_env_path = os.path.join(
        os.path.dirname(os.path.abspath(__file__)), "autorino_env_default.yml"
    )

    # Determine the environment file path
    envfile_path_use = (
        envfile_path or os.environ.get("AUTORINO_ENV", "")
    )

    if not envfile_path_use or not os.path.exists(envfile_path_use):
        logger.warning(
            f"{'Custom environment configfile not defined' if not envfile_path_use else f'File not found: {envfile_path_use}'}"
        )
        logger.warning("Falling back to default values in %s", default_env_path)
        envfile_path_use = default_env_path

    logger.info("Loading environment configfile: %s", envfile_path_use)

    # Load and merge environment configurations
    env_default = yaml.safe_load(open(default_env_path))["environment"]
    env_custom = yaml.safe_load(open(envfile_path_use))["environment"]
    return mergedeep.merge({}, env_default, env_custom)

# Global environment dictionary used throughout the package
ARO_ENV_DIC = read_env()